/*      file     : Planner/dijkstra.cc
        purpose  : object oriented dijkstra
                   to be able to plug in different types of priority queue
        author   : Riko Jacob
                   interface changes and additions by Marcus Rickert
        RCS      : $Id: dijkstra.cc,v 1.9 2005/03/25 12:27:31 mr Exp $

        Copyright (C) 2005 Riko Jacob and Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#include <unistd.h>
#ifdef LINUX
#  include <getopt.h>
#endif

#include "synchronize.h"

// one RE, no union, implicit + at all symbols

// to start with. quick and dirty but efficient. 
// to be able to try things as soon as possible.

// Graphs are stored as an array of nodeinfos, that has an array of neighbours
// because of that we need this constants right here
//#define MAXNODES 10000
#define MAXDEGREE 8
// For paths
#define MAXLENGTH 1000

#define MAXBINS 28

#define BIN_WIDTH 900
#define START_HOUR 5

double adj_par = 1.0;

#define ROUTE_ID_RANDOM_RANGE 1000

long NextRouteID = ROUTE_ID_RANDOM_RANGE;
  
// ===================================================================================

struct TLevel0Data

{
    double Ratio;
    double Weight;
    double Factor;
};

// ===================================================================================

#define DIJKSTRA_OPTIONS "ab:c:d:f:F:g:i:ln:I:O:p:r:Rtv:z"

#define DEFAULT_REPLANNING_FRACTION    0.1
#define DEFAULT_REDUCTION_FRACTION     0.0
#define DEFAULT_MAX_NUMBER_OF_PLANS    350000
#define DEFAULT_INITIAL_AGE            0
#define DEFAULT_PIPE_NAME              0
#define DEFAULT_BINARY_MODE            0

#define DEFAULT_INPUT_FILENAME     "plans.input.txt"
#define DEFAULT_OUTPUT_FILENAME    "plans.output.txt"

#define NODE_EXTENSION             ".nod"
#define EDGE_EXTENSION             ".edg"

TLevel0Data Level0Data[MAXBINS];

long NrOfNodes;

// '-n': maximum number of plans
int numb_plans = DEFAULT_MAX_NUMBER_OF_PLANS;

// '-f': replanning fraction of all plans (also see reduce_frac)
float repl_frac = DEFAULT_REPLANNING_FRACTION;

// '-d': replanning fraction of plans that have never been replanned, if non-zero
// '     repl_frac' only applies to those plans that have been replanned at least once
float reduce_frac = DEFAULT_REDUCTION_FRACTION;

// '-r': random seed, if -1 don't randomize
long random_seed = -1;

// '-i': current iteration
int iteration = 1;

// '-t': update travel time estimates
int update_estimates = 0;

// '-a': update age stamps
int update_age = false;

// '-g': initial age
int initial_age = DEFAULT_INITIAL_AGE;

// '-l': linear age-dependant selection
int linear_age_dependant = 0;

// '-I': input filename
char *input_filename = DEFAULT_INPUT_FILENAME;

// '-O': output filename
char *output_filename = DEFAULT_OUTPUT_FILENAME;

// '-F': link feedback filename
char *feedback_filename = 0;

// '-p': pipe-name
char *pipe_name = DEFAULT_PIPE_NAME;
TSyncInfo SyncInfo;

// '-v': network base filename

char *network_base_filename = "./TRANSIMS-Casestudy-1C";

// '-b': binary mode
//  0: input+output ascii
//  1: input binary, output ascii
//  2: input ascii, output binary
//  3: input+output, binary
int binary_mode = DEFAULT_BINARY_MODE;

int OutputBinary = 0;
int InputBinary = 0;

// '-z': level-0 correction

enum TLevel0 { Level0Off = 0, Level0Linear, Level0SquareRoot };

TLevel0 level0 = Level0Off;

// 'R': renumber route IDs

int renumber_route_id = 0;

// ===================================================================================

extern "C"
{
    struct TRouteEntry
    {
	long ID;
	long Estimate;
    };

    struct TRoute	
    {
	long planNo;
	long mode;
	long vehicleID;
	long driverPlanNo;
	long startAcc;
	long steps;
	TRouteEntry Entries[MAXLENGTH];
    };
}

// ===================================================================================

// #define INFTY (MAXFLOAT+MAXFLOAT)
/*
#include "/opt/SUNWspro/include/sunmath.h" 

const float INFTY = infinity();
*/

#define	FLT_MAX		3.402823466E+38F  /* max decimal value of a "float" */

#define INFTY FLT_MAX

#define true -1
#define false 0

#undef sedge
#define sv true
#define dout false
#undef statistic
// twoway can currently not work on time dependent things
#define  oneway
#undef dataout 

#undef  debug
#undef  printroutes
#undef verbose

#undef UseNFA

// Everything is stored in constant size hashes, so we have to provide the sizes as powers of two
#define NODEHASH 14
#define MAXNODES 16384
#define LINKHASH 15

// We don't allways want all the plans.
// became input parameter, this is only default now
#define NUMBEROFPLANS 1000

#include<stdlib.h>      // only for CC because of exit()
#include<iostream.h>    // for easy output
#include<stdio.h>       // for file reading
#include<math.h>        // for euclidian distances

// different things in different files
#include"heap.cc"   // provides  p_queue_heap
#include"hash.cc"   // provides  hash


#define sqr(x) ((x)*(x))
#define increase(a) ( (a) = ( (a)<'z' ? (a)+1 : 'a' ) )


// for regular expr
#define NFAlength 2
char nullNFA[NFAlength] = "c";

float maxspeed =0;
float scale = 10000, maxscale =0;

int do_output = dout;
int do_sv = sv;

// classes to be hashed have to provide their id. This is not especially efficient
// in terms of time if you do a lot of hashing, but here, 
// space efficiency is more important, as we rarely have to access things through the hash.
// May be it is a little inconsistent to have things stored in a hash and access them by 
// pointers directly.

// here we say there must exist an int class.id
float *TravelTimes;
// until (including) 900: bin 0, etc
// inline

int TimeBin(int time)

{
    int t = (time - 1) / 900;
    return (t<0? 0:(t>=MAXBINS? MAXBINS-1:t));
}

float & TrTime( int node, int pos, int time ) 

{ 
   return TravelTimes[(node*MAXBINS*MAXDEGREE) + (pos*MAXBINS) + TimeBin(time)];
};

float & TrTimeByBin( int node, int pos, int bin ) 

{ 
   return TravelTimes[(node*MAXBINS*MAXDEGREE) + (pos*MAXBINS) + bin];
};

class nodeinfo
{
public:
  nodeinfo( int th_id = -1, float xx=0, float yy=0 ) :
    id(th_id), x(xx), y(yy)
  { 
    
    int i,j; 
    for(j=0;j<2;j++) 
      {
	//Xmin_dist[j]=0; Xreached_by[j]=(NoHashIndex); Xexpand[j]=-1;
	Xdegree[j]=0;
	for(i=0;i<MAXDEGREE;i++) Xneighbours[j][i]= NoHashIndex; 
	for(i=0;i<MAXDEGREE;i++) Xdistances[j][i]=0.001; 
	for(i=0;i<MAXDEGREE;i++) Xlinks[j][i]= NoHashIndex; 
      }
  };
  void insert_neighbour( hashindex nodei, hashindex linki ,float dst, char mode );
  int is_void()    { return Xdegree[0] == -1; };
  void make_void() { Xdegree[0] = -1;};

  int       id;
  float     x,y;

  short int Xdegree[2];              // these model the graph
  hashindex Xneighbours[2][MAXDEGREE];
  float     Xdistances[2][MAXDEGREE];
  hashindex Xlinks[2][MAXDEGREE];
};

// +TODO+ change to Horner-scheme to evaluate a polynomial
inline int position( hashindex nindx, int nfapos, int direction =0 )
{  return direction*(MAXNODES*NFAlength) + nindx*NFAlength + nfapos; };

float *Xmin_dist;
hashindex *Xreached_by_nw;
hashindex *Xreached_by_link;
int *Xreached_by_nfa;
int *generation;

// +TODO+ timing between generation and smart cleanup

#define arraysize (MAXNODES*NFAlength*2)

class linkinfo
{
public:
  linkinfo( int i =-1, hashindex a= NoHashIndex, hashindex b= NoHashIndex ) : 
    id(i), NodeA(a), NodeB(b) {};
  void make_void() { id = -2; NodeA=(hashindex)-2; NodeB=(hashindex)-2;}; // -2 not impossible...
  int is_void() { return id == -2 && NodeA==(hashindex)-2 && NodeB==(hashindex)-2;};

  int id;
  hashindex NodeA, NodeB;
  float weight;
  int mode;
  char LanesA, LanesB;
};

// this is conceptually not nice, but it occured to be the esiest way 
// to provide tis method with its index in the hash
// determing this index would lead to some other inconsistencies
void nodeinfo::insert_neighbour( hashindex nodei, hashindex linki ,float dst, char mode )
{
  if ( dst < 0.00000001 ) cout << "strange link distance: "<< dst << endl; // linkid is not accesible
  // insert link in direction mode
  if ( Xdegree[mode] >= MAXDEGREE ) exit(7); // degree limit exceeded
  Xneighbours[mode][Xdegree[mode]]=nodei;
  Xdistances[mode][Xdegree[mode]] =dst;
  Xlinks[mode][Xdegree[mode]]     =linki;
  
  int i;                      // searching for doubled links
  for(i=0;i<Xdegree[mode];i++ ) 
    if ( Xneighbours[mode][i] == nodei) 
      cout << "doubled link" << Xlinks[mode][i] << endl;
  // this used to be more convenient when things were simply pointers

  Xdegree[mode]++;
  return;
}

class implicit_node
{
public:
  implicit_node(hashindex ha, int nfa) : nw_idx(ha), nfa_pos(nfa) {};
  implicit_node() {};
  hashindex nw_idx;
  int  nfa_pos;
};

#ifdef printroutes
float maxx=0.0,maxy=0.0,minx=0.0,miny=0.0;

void coords(FILE *f, float x, float y)
{
  if ( x > maxx || maxx == 0.0) maxx = x;
  if ( y > maxy || maxy == 0.0) maxy = y;
  if ( x < minx || minx == 0.0) minx = x;
  if ( y < miny || miny == 0.0) miny = y;
  fprintf( f, " %f %f \n", x, y);
}
char countc='a';

char coordname[]="Dat/crd.a";

void pr_coords()
{
  coordname[8]=countc;
  FILE *co=fopen(coordname, "w");
  float xx=(maxx+minx)/2;
  float yy=(maxy+miny)/2;
  float dx = maxx-minx;
  float dy =maxy-miny;
  float dd = ( dx < dy ? dy : dx ) *.6;  

  //  fprintf( co," %f %f \n %f %f \n\n", maxx, maxy, minx, miny); 
  
  fprintf( co," set xrange[%f:%f]\n set yrange[%f:%f]\n", xx-dd, xx+dd, yy-dd, yy+dd ); 
  fclose(co);

  maxx=0.0;maxy=0.0;minx=0.0;miny=0.0;
};

char routname[]="Dat/rta.a";
char debname[]="Dat/dba.a";
#endif

int act_gen =0;

// ===================================================================================

#include "dijkstra.h"

// ===================================================================================

p_queue_heap<implicit_node> Xdijk_heap[2] = { MAXNODES*NFAlength, MAXNODES*NFAlength };
int dijkstra( int starttime, hashindex startlink, hashindex destlink, hashindex *links, float *dists, 
	      hash<nodeinfo> &node_hash, hash<linkinfo> &link_hash, char *NFA, char *bnam ) 
{
  //  cout << "-----" << starti << " : " << desti << endl;
  int i,j,xpos,xstate,nstate;
#ifdef printroutes
  for(i=0;bnam[i]; i++) routname[4+i]=bnam[i];
   routname[8]=countc;
#endif
#ifdef debug
   FILE *deb[NFAlength];
   debname[6]='a';
   for(i=0;i<NFAlength;i++)
     {
       deb[i]=fopen( debname, "w");
       debname[6]++;
     }
#endif

  act_gen+=2;

  if ( startlink == destlink )
    {
      links[0]=startlink;dists[0]=0.0;
      return 0;
    }

  Xdijk_heap[0].clear();
  Xdijk_heap[1].clear();
  
  hashindex startnode;

  i=0;
  if( link_hash[startlink].LanesB > 0 )
    {
      startnode = link_hash[startlink].NodeA;
//   int doit=true;
//    for( i=0; NFA[i]!=0; i++ )
//     {
//       if( doit )
// 	{
          Xdijk_heap[0].insert( implicit_node( startnode, i ), 0.0 );
	  xpos=position( startnode, i);
	  Xmin_dist[xpos]=0.0;
	  Xreached_by_nfa[xpos]=-1;
	  Xreached_by_nw [xpos]=NoHashIndex;
	  generation[xpos] = act_gen-1;
// 	}
//       doit=(NFA[i]=='+');
//     }
    }
      
  if( link_hash[startlink].LanesA > 0 )
    {
      startnode = link_hash[startlink].NodeB;

//   int doit=true;
//   for( i=0; NFA[i]!=0; i++ )
//     {
//       if( doit )
// 	{
 	  Xdijk_heap[0].insert( implicit_node( startnode, i ), 0.0 );
	  xpos=position( startnode, i);
	  Xmin_dist[xpos]=0.0;
	  Xreached_by_nfa[xpos]=-1;
	  Xreached_by_nw [xpos]=NoHashIndex;
	  generation[xpos] = act_gen-1;
// 	}
//       doit=(NFA[i]=='+');
//     }
    }
  implicit_node act_node;
  const  int phase = 0;  // takes care of alternation between the two ways, begin with start
  //#define oneway

#ifdef sedge
  float destx, desty;
  destx= 0.5*(node_hash[link_hash[destlink].NodeA].x + node_hash[link_hash[destlink].NodeB].x);
  desty= 0.5*(node_hash[link_hash[destlink].NodeA].y + node_hash[link_hash[destlink].NodeB].y);
#endif

  int done = true;
  while( 1 )  // we will use a break at an apropriate point of the loop
    {
      if ( Xdijk_heap[phase].empty() ) {done =false; break;}  // only happens if no path exists
      act_node = Xdijk_heap[phase].min_member();
      nodeinfo &a_node=node_hash[act_node.nw_idx];
      xstate=act_node.nfa_pos;
      xpos=position(act_node.nw_idx, xstate, phase);
      float act_dist = Xmin_dist[xpos];
      //      printf("%f %f %d %d %d \n", Xdijk_heap[phase].min_dist(), act_dist, act_node.nw_idx, xstate, xpos );
      Xdijk_heap[phase].remove_min();
      //      cout << "look at " << node_hash[act_nodei].id << endl;
      //      printf(" %d %d %d %d \n", Xdijk_heap[phase].get_size(), act_node.nw_idx, xstate, xpos );
      if ( generation[xpos] != act_gen+1 ) // otherwise node is already expanded
	{
	  generation[xpos] = act_gen+1;

#ifdef debug	  
	  coords( deb[xstate], a_node.x, a_node.y );
	  coords( deb[xstate], node_hash[Xreached_by_nw[xpos]].x,  node_hash[Xreached_by_nw[xpos]].y);
	  fprintf( deb[xstate], "\n" );
#endif
	  //	  printf(  " %f %f \n", a_node.x, a_node.y );
	  // here we assume accessories to be exactly in the middle of the link.
	  if( act_node.nw_idx == link_hash[destlink].NodeA
	      && link_hash[destlink].LanesA > 0
	      && NFA[xstate+1] ==0 ) break;
	  if( act_node.nw_idx == link_hash[destlink].NodeB
	      && link_hash[destlink].LanesB > 0
	      && NFA[xstate+1] ==0 ) break;


	  hashindex act_ngb;
	  for( i = 0 ; i < a_node.Xdegree[phase]; i++ )  
	    // make neighbours to fringe nodes and test
#ifdef UseNFA
	    for(j=0; j<2;j++)
	    {
	      linkinfo &ngb_link=link_hash[a_node.Xlinks[phase][i]];
	      nstate=xstate+j;

	      if( NFA[nstate] == 0 ) continue;
	      if( '0' <= NFA[nstate] && NFA[nstate] <= '9' && NFA[nstate]-'0' != ngb_link.mode ) continue;
	      if( 'A' <= NFA[nstate] && NFA[nstate] <= 'Z' && NFA[nstate]-'A' == ngb_link.mode ) continue;
	      if( NFA[nstate] == 'f' && ngb_link.mode != 1 && ngb_link.mode != 6 ) continue;
	      if( NFA[nstate] == 'b' && ngb_link.mode != 2 && ngb_link.mode != 3 ) continue;
	      if( NFA[nstate] == 'w' && 
		  ( ngb_link.mode ==0 || ngb_link.mode == 1 || ngb_link.mode == 6) ) continue;
#endif
#ifndef UseNFA
	      {
		nstate =xstate;
#endif
	      float new_dist= TrTime( act_node.nw_idx, i, int( act_dist+starttime ) );
	      //	      printf("i%f", new_dist);
	      if( new_dist == INFTY ) new_dist = a_node.Xdistances[phase][i];
	      //	      printf("|%fi", new_dist);
	      if (new_dist <= 0.0 ) exit(17);
	      new_dist += act_dist;
	      act_ngb=a_node.Xneighbours[phase][i];
	      //	      nodeinfo &ngb=node_hash[act_ngb];
	      int ngbpos=position(act_ngb, nstate,phase);
	      //	      printf(" %f %f %f %d %d %d \n", act_dist, Xmin_dist[ngbpos], new_dist, act_ngb, nstate, ngbpos );
	      if ( act_gen > generation[ngbpos] 
		   || new_dist <= Xmin_dist[ngbpos] )  // normal update necessary?
		{
		  Xmin_dist[ngbpos] = new_dist;
		  Xreached_by_nw[ngbpos] = act_node.nw_idx;
		  Xreached_by_link[ngbpos] = a_node.Xlinks[phase][i];
		  Xreached_by_nfa[ngbpos] = xstate;
#ifdef sedge
		  nodeinfo &newnode = node_hash[act_ngb];
		  new_dist += scale * sqrt( sqr(newnode.x-destx)+sqr(newnode.y-desty)); 
#endif
		  Xdijk_heap[phase].insert( implicit_node (act_ngb,nstate), new_dist ); 
		  if (generation[ngbpos]<act_gen) generation[ngbpos]=act_gen;
		}
	    }
	}  // end if: was dummy?
    }
#ifdef debug
  for(i=0;i<NFAlength;i++) fclose(deb[i]);
  increase( debname[8] );
#endif
  int step =0;
  if( !done )
    {
      cout << "no path exsting" << endl;
      step = -1;
    }
  else
    {  
      //      cout << "path found" << endl;
#ifdef printroutes
      FILE *routFP=fopen( routname, "w" );
      nodeinfo &snode=node_hash[act_node.nw_idx];
      coords( routFP, snode.x, snode.y);
#endif
      while( step == 0 || Xmin_dist[xpos] != 0.0 ) // goes normally to one end of starting link
	{
	  //output: node,x,y
	  dists[step] = Xmin_dist[xpos];
	  links[step] = Xreached_by_link[xpos];
	  
	  int oldstate = xstate;

	  xstate  = Xreached_by_nfa[xpos];
	  hashindex a_node = Xreached_by_nw[xpos];
#ifdef printroutes
	  nodeinfo &wnode=node_hash[a_node];
	  coords( routFP, wnode.x, wnode.y);
	  //	  printf( " %f   %f \n", wnode.x, wnode.y);

	  //	  printf( " %d %d %f \n", xstate, a_node, dists[step]);
 	  if( xstate != oldstate )
 	    {
 	      fclose(routFP);
 	      increase( routname[6] );
 	      routFP = fopen( routname, "w" );
 	      nodeinfo &enode=node_hash[a_node];
 	      fprintf( routFP, " %f   %f \n", enode.x, enode.y);
 	    }
#endif
	  xpos=position(a_node, xstate );
	  step++;
	}
      links[step] = startlink;
      dists[step] = 0;
#ifdef printroutes
      fclose(routFP);
#endif
    }

  //  cleanup( Xdijk_heap, used_node_info, number_used, node_hash );
  //  cout << "dijkstra"<<endl;
  return step;
}

void ReadNodeFile( hash<nodeinfo> &node_hash )
{
  int    id=0,oldid=-1;
  float  x,y;
  char   filename[256];
  FILE  *nodeFP;

  sprintf(filename, "%s%s", network_base_filename, NODE_EXTENSION);
  nodeFP = fopen(filename, "r");
  printf("Reading node file '%s'...\n", filename);
  if(!nodeFP){
    printf("Could not open %s!\n", filename);
    exit(3);
  }
  while ( (getc( nodeFP )) != 10 ) ;  // skips Headerline

  while( 1 )
    {
      oldid =id;
      fscanf(nodeFP, "%d, %f, %f", &id, &x, &y);
      if ( oldid == id ) break;
      node_hash.insert( nodeinfo( id, x,y));
    };
  return;
}

// ===================================================================================

void ReadLinkFile( hash<nodeinfo> &node_hash, hash<linkinfo> &link_hash )
{
  FILE *linkFP;
  int   oldid=-1;
  int   LinkID=0,NodeA,NodeB,PermLanesA,PermLanesB,FClass,TollAB,TollBA;
  float Length,CapacityA,CapacityB,SpeedLmtA,SpeedLmtB,FreeSpdA,FreeSpdB,CrawlSpdA,CrawlSpdB;
  char  filename[256];

  sprintf(filename, "%s%s", network_base_filename, EDGE_EXTENSION);
  linkFP = fopen(filename, "r");
  if(!linkFP){
    printf("Could not open %s!\n", filename);
    exit(4);
  }
  printf("Reading link file '%s'...\n", filename);

 while ( (getc( linkFP )) != 10 ) ;  // skips Headerline

 while( 1 )
   {
     oldid=LinkID;
     fscanf(linkFP, "%d, %d, %d, %d, %d, %f, %f, %f, %f, %f, %f, %f, %f, %f, %d, %d, %d",
	  &LinkID,&NodeB,&NodeA,&PermLanesA,&PermLanesB,&Length,&CapacityA,&CapacityB,
	  &SpeedLmtA,&SpeedLmtB,&FreeSpdA,&FreeSpdB,&CrawlSpdA,&CrawlSpdB,&FClass,
	  &TollAB,&TollBA);
     if ( oldid == LinkID ) break;

     hashindex A_ind = node_hash.index(NodeA);
     hashindex B_ind = node_hash.index(NodeB);

     nodeinfo &A_info = node_hash[A_ind] ;
     nodeinfo &B_info = node_hash[B_ind] ;
     float mydist = sqrt( sqr(A_info.x-B_info.x) + sqr(A_info.y-B_info.y) );
//       if ( this_scale > maxscale ) 
//         {
//  	 maxscale = this_scale;
 // 	 cout << LinkID << " - " << Length << "/" << mydist << "  : " << this_scale << endl;
      //        }
      //      Cout << Linkid << "    \t" << FClass << "\t" << NodeA <<"\t" << NodeB << "\t" << 
      //   Length << "\t" << mydist << "\t" << this_scale << endl;
     //scale = 1;
     linkinfo thislink( LinkID, A_ind, B_ind );
#ifdef detour
     if( LinkID == detour ) FClass = 9;
#endif
     thislink.mode=FClass;
     thislink.LanesA=PermLanesA;
     thislink.LanesB=PermLanesB;
     thislink.weight=Length;
     hashindex newlinkindex = link_hash.insert( thislink );

     if ( PermLanesA > 0 )
       {
	 if ( A_info.is_void() ) exit(8); // unknown node used
 	 A_info.insert_neighbour( B_ind, newlinkindex, Length/FreeSpdA, 0);
 	 B_info.insert_neighbour( A_ind, newlinkindex, Length/FreeSpdA, 1);
	 float this_scale = (Length/FreeSpdA) / mydist;
	 //	 if ( this_scale < 0.5 ) { Length = 0.5 * mydist; this_scale =0.5; }
	 //  this is a real bad data-bug-fixer !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 if ( this_scale < scale ) 
	   {
	     scale = this_scale;
	   }
	 //	 if (FreeSpdA > maxspeed) maxspeed = FreeSpdA;
       }

     if ( PermLanesB > 0 )
       {
	 if ( B_info.is_void() ) exit(8); // unknown node used
 	 B_info.insert_neighbour( A_ind, newlinkindex , Length/FreeSpdB, 0);
 	 A_info.insert_neighbour( B_ind, newlinkindex , Length/FreeSpdB, 1);
	 //	 if (FreeSpdB > maxspeed) maxspeed = FreeSpdB;
	 float this_scale = (Length/FreeSpdB) / mydist;
	 //	 if ( this_scale < 0.5 ) { Length = 0.5 * mydist; this_scale =0.5; }
	 //  this is a real bad data-bug-fixer !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 if ( this_scale < scale ) 
	   {
	     scale = this_scale;
	   }
       }
    };
 // cout<<endl;
 return;
}

// ===================================================================================

int trytime;
void 
ReadFeedback(  hash<nodeinfo> &node_hash, hash<linkinfo> &link_hash)
{
  hashindex FromIndex;
  TravelTimes = new float[ MAXNODES * MAXDEGREE * MAXBINS ];
  if( 0 == TravelTimes ) exit(38);
  
  int i,j,k;
  for(i=0; i< MAXNODES * MAXDEGREE * MAXBINS; i++) TravelTimes[i]=INFTY;

  for( i=0; i< MAXNODES; i++)
    {
      if( node_hash[i].id != -1 )
	{
	  for(j=0; j < node_hash[i].Xdegree[0] ; j++ ) 
	    {
	    for(k=0; k < MAXBINS; k++ )
	     {
	       TravelTimes[(i*MAXBINS*MAXDEGREE) + (j*MAXBINS) +k] =
		   (level0 == Level0Off)? node_hash[i].Xdistances[0][j] :
		       - node_hash[i].Xdistances[0][j];  // already traveltime for free flow
	     }
	    }
	}
    }
  
  int oldid=-1;
  int LinkID=0,NodeID=0,Count,Time;
  float Sum, SumSquares;

  if (feedback_filename == 0) {
      printf("WARNING: no feedback file specified; using free speeds.\n");
      return;
  }

  FILE *feedbFP = fopen(feedback_filename, "r");
  if(!feedbFP){
    printf("Could not open %s!\n", feedback_filename);
    exit(5);
  }

  printf("Reading link feedback from file '%s'...\n", feedback_filename);

 while ( (getc( feedbFP )) != 10 ) ;  // skips Headerline
 
 int time=0;

 NrOfNodes = 0;
 while( 1 )
   {
     //     oldid=LinkID+NodeID;
     if( EOF == fscanf(feedbFP, "%f %d %d %f %d %d",
	    &Sum, &Time, &NodeID, &SumSquares, &Count, &LinkID ) ) break;
//      printf( "%f %d %d %f %d %d\n",
// 	    Sum, Time, NodeID, SumSquares, Count, LinkID );
     //     if ( oldid == LinkID+NodeID ) break;
     NrOfNodes++;
     FromIndex = node_hash.index( NodeID );
     nodeinfo &FromNode = node_hash[FromIndex];


     //     if ( time != Time ) printf("%d%\n", time=Time);

     for( i=0; i< FromNode.Xdegree[0] ; i++ )
       if( link_hash[FromNode.Xlinks[0][i]].id == LinkID )
	 {
	   float olddelay;

	   olddelay = TrTime( FromIndex, i, Time );
	   if (olddelay < 0)
	       olddelay = -olddelay;

// 	   if( trytime == Time )
// 	     printf(" %f %f \n %f %f \n\n",
// 		  FromNode.x, FromNode.y,
// 		  node_hash[FromNode.Xneighbours[0][i]].x,
// 		  node_hash[FromNode.Xneighbours[0][i]].y );
// 	   printf( "%f %d %d %f %d %d\n",
// 		   Sum, Time, NodeID, SumSquares, Count, LinkID );
//   	   printf( "nodeid %d\tnodehash %d\toldspeed %f\tnewspeed %f\n",
//   		   NodeID, FromIndex, oldspeed, Sum / Count ); 
	   float delay =  Sum / Count ;
	   if (level0)
	   {
	       double Ratio;
	       double Weight;

	       Weight = link_hash[FromNode.Xlinks[0][i]].weight;
	       Ratio = delay / olddelay;
	       Level0AddFeedback(TimeBin(Time), Ratio, Weight);
	   }
	   TrTime( FromIndex, i, Time ) =  delay;
	   if( delay == INFTY ) printf( "INFTY feedback: link %d, time %d\n",LinkID,Time);
// 	   if ( TrTime( FromIndex, i, Time ) != Sum/Count ) 
// 	     printf( " %f %f \n", TrTime( FromIndex, i, Time ), Sum/Count );
	  break;
	 };
     if ( i == FromNode.Xdegree[0] ) printf( " link id %d not found in outgoing links of from node\n", LinkID);
   }
     
}

// ===================================================================================

FILE *planFP;
FILE* outFP;

void OpenPlanFile()
{
  char header[201];
  planFP = fopen(input_filename, "r");
  if(!planFP){
    printf("Could not open %s!\n", input_filename);
    exit(5);
  }

  printf("Reading plans from file '%s'...\n", input_filename);

  if (!InputBinary)
      fgets( header , 200 , planFP ) ;

      /* write header to output file */
  outFP = fopen(output_filename, "w");

  printf("Writing plans to file '%s'...\n", output_filename);

  if (!OutputBinary)
      if (InputBinary)
	  fprintf(stderr, "WARNING: reading binary and writing ascii! Header will be missing!\n");
      else
	  fprintf(outFP, "%s\n" , header);
}

char nfa[] = "c";

float adj_fraction( int age, float mypar, float repl)
{
  return mypar*repl*(age+1);
}

// ===================================================================================

void
Level0AddFeedback(int TimeBin, double Ratio, double Weight)

{
    //    fprintf(stderr, "bin %ld ratio=%f weight=%f\n", TimeBin, Ratio, Weight);

    Level0Data[TimeBin].Ratio += Weight * Ratio;
    Level0Data[TimeBin].Weight += Weight;
}

// ===================================================================================

int 
Level0ComputeCorrections(hash<nodeinfo> &node_hash)

{
    int     CBin, CNode, CInc;
    FILE   *Output;

    if ((Output = fopen("level0.txt", "w")) == 0)
    {
	fprintf(stderr, "cannot open level0 stat file\n");
	return 0;
    }
    for (CBin = 0; CBin < MAXBINS; CBin++)
    {
	if (Level0Data[CBin].Weight > 0.01)
	{
	    Level0Data[CBin].Factor = Level0Data[CBin].Ratio / Level0Data[CBin].Weight;
	    if (level0 == Level0SquareRoot)
		Level0Data[CBin].Factor = sqrt(Level0Data[CBin].Factor);
	    fprintf(stderr, "correction for time-bin %2d : %6.2f\n",
		    CBin, Level0Data[CBin].Factor);
	}
	else
	{
	    fprintf(stderr, "no level-0 correction factor for bin %d\n", CBin);
	}	    
	for (CNode = 0; CNode < MAXNODES; CNode++)
	    if (node_hash[CNode].id != -1 )
		for (CInc = 0; CInc < node_hash[CNode].Xdegree[0]; CInc++) 
		    if (TrTimeByBin(CNode, CInc, CBin) < 0)
			TrTimeByBin(CNode, CInc, CBin) *= -Level0Data[CBin].Factor;
	fprintf(Output, "%d %f %f\n", START_HOUR * 3600 + CBin * BIN_WIDTH,
		(double) (START_HOUR * 3600 + CBin * BIN_WIDTH) / 3600.0,
		Level0Data[CBin].Factor);
    }    
    fclose(Output);

    // just a check...

    for (CNode = 0; CNode < MAXNODES; CNode++)
	if (node_hash[CNode].id != -1)
	    for (CInc = 0; CInc < node_hash[CNode].Xdegree[0] ; CInc++)
		for (CBin = 0; CBin < MAXBINS; CBin++)
		    if (TravelTimes[(CNode*MAXBINS*MAXDEGREE) + (CInc*MAXBINS) + CBin] < 0)
		    {
			fprintf(stderr, "negative weight found!");
			exit(111);
		    }
    return 1;
}

// ===================================================================================

int
Level0Prepare(void)

{
    int CBin;

    for (CBin = 0; CBin < MAXBINS; CBin++)
    {
	Level0Data[CBin].Factor = 0.0;
	Level0Data[CBin].Weight = 0.0;
	Level0Data[CBin].Factor = 1.0;
    }
    return 1;
}

// ===================================================================================

int
ParseOptions(int argc, char *argv[])

{
  int  c;
  int  ParameterNr;

  //  extern char *optarg;
  //  extern int   opterr;
  //  extern int   optind;

  opterr = 0; /* prevent errors */
  do
    {
      switch(c=getopt(argc, argv, DIJKSTRA_OPTIONS))
	{
	case EOF:
	  break;
	  
	case 'a':
	  update_age = true;
	  break;

	case 'b':
	  binary_mode = atoi(optarg);
	  break;

	case 'd':
	  reduce_frac = atof(optarg);
	  break;

	case 'f':
	  repl_frac = atof(optarg);
	  break;

	case 'g':
	  initial_age = atoi(optarg);
	  break;

	case 'i':
	  iteration = atoi(optarg);
	  break;

	case 'l':
	  linear_age_dependant = true;
	  break;

	case 'n':
	  numb_plans = atoi(optarg);
	  break;

	case 'I':
	  input_filename = optarg;
	  break;

	case 'O':
	  output_filename = optarg;
	  break;

	case 'F':
	  feedback_filename = optarg;
	  break;

	case 'p':
	  pipe_name = optarg;
	  break;

	case 'r':
	  random_seed = atol(optarg);
	  break;

	case 'R':
	  renumber_route_id = true;
	  break;

	case 't':
	  update_estimates = true;
	  break;

	case 'v':
	  network_base_filename = optarg;
	  break;

	case 'z':
	  level0 = (TLevel0) atoi(optarg);
	  break;

	case '?':
	case ':':
	  fprintf(stderr, "WARNING: ignoring option '%c'\n", c);
	  break;
	  
	default:
	  fprintf(stderr, "WARNING: option '%c' not handled\n", c);
	}
    }
  while (c != EOF);
  ParameterNr = 1;
  while (optind < argc)
  {
      fprintf(stderr, "WARNING: %d. parameter '%s' not handled\n", ParameterNr, argv[optind]);
      ParameterNr++;
      optind++;
  }
  InputBinary = binary_mode & 1;
  OutputBinary = binary_mode & 2;
  return true;
}

// ===================================================================================

int
ReadRoute(FILE *in, TRoute *Route)

{
    if (InputBinary)
    {
	fread((void *) &Route->planNo, sizeof(long), 6, in);
	if (feof(in))
	    return 0;
	fread((void *) &Route->Entries[0].ID, sizeof(long), 2*(Route->steps+1), in);
    }
    else
    {
	long CStep;

	if (6 != fscanf(in, "%ld %ld %ld %ld %ld %ld", 
			&Route->planNo, &Route->mode, &Route->vehicleID, 
			&Route->driverPlanNo, &Route->startAcc, &Route->steps))
	    return 0;
	for (CStep = 0; CStep < Route->steps + 1; CStep++)
	    fscanf(in, "%ld %ld ", &Route->Entries[CStep].ID, &Route->Entries[CStep].Estimate);
    }
    return 1;
}

// ===================================================================================

void
WriteRoute(FILE *out, TRoute *Route)

{
    if (renumber_route_id)
    {
	Route->planNo = NextRouteID + (long) (drand48() * ROUTE_ID_RANDOM_RANGE);
	NextRouteID += ROUTE_ID_RANDOM_RANGE;
    }
    if (OutputBinary)
    {
	fwrite((void *) &Route->planNo, sizeof(long), 6, out);
	fwrite((void *) &Route->Entries[0].ID, sizeof(long), 2*(Route->steps+1), out);
    }
    else
    {
	long CStep;

	fprintf(out, "%ld %ld %ld %ld %ld %ld\n", Route->planNo, Route->mode, 
		Route->vehicleID, Route->driverPlanNo, Route->startAcc, Route->steps);
	for (CStep = 0; CStep < Route->steps + 1; CStep++)
	    fprintf(out, " %ld %ld", 
		    Route->Entries[CStep].ID, Route->Entries[CStep].Estimate);
	fprintf(out, "\n");
    }
}

// ===================================================================================

int 
main(int argc, char **argv)
{
//   char tfiles[]="Dat/test.a";
//   char rfiles[]="Dat/route.a";

//   FILE *statistics = fopen( "Dat/statistics", "w" );

  // <<<< MaRi
  long RoutesInFile = 0;
  long RoutesReplanned = 0;
  // >>>>

  if (! ParseOptions(argc, argv))
    {
      fprintf(stderr, "\nERROR: cannot parse options\n");
      exit (2);
    }

  printf("network base filename          : %s\n", network_base_filename);
  printf("input file                     : %s\n", InputBinary? "binary":"ascii");
  printf("output file                    : %s\n", OutputBinary? "binary":"ascii");
  printf("sync pipe                      : %s\n", pipe_name? pipe_name:"not active");
  printf("iteration                      : %d\n", iteration);
  printf("random seed                    : %ld\n", random_seed);
  printf("max number of plans            : %d\n", numb_plans);
  printf("replanning fraction            : %f\n", repl_frac);
  printf("reduce fraction                : %f\n", reduce_frac);
  printf("linear age-dependant selection : %s\n", linear_age_dependant? "yes":"no");
  printf("initial route age              : %d\n", initial_age);
  printf("update age                     : %s\n", update_age? "yes":"no");
  printf("update travel time estimates   : %s\n", update_estimates? "yes":"no");
  printf("renumber route IDs             : %s\n", renumber_route_id? "yes":"no");
  printf("level-0 correction             : %s\n", 
	 (level0 == Level0Linear)? "linear":((level0 == Level0SquareRoot)? "square root":"no"));

  SyncInit(&SyncInfo);

  if (pipe_name)
    {
      fprintf(stderr, "waiting for sync with route converter...\n");
      if (!SyncActivate(&SyncInfo, pipe_name, 1))
      {
	  fprintf(stderr, "cannot activate sync pipe\n");
	  exit(3);
      }
      fprintf(stderr, "synchronized with route converter;\n");
    }
  if (random_seed != -1)
    srand48( random_seed );
  
  if (linear_age_dependant)
    {
      double p2;

      p2 = repl_frac * repl_frac;
      adj_par = 0.5 * M_PI *p2 * exp(0.25 * M_PI * p2);
      printf("effective selection factor     : %f\n", adj_par);
    }

//   char *NFA;
//   if( argc > 2 ) NFA=argv[2];
//   else NFA=nfa;

//   int numb_skip=0;
//   if( argc > 3 ) numb_skip = atoi( argv[3] );

//   if( argc > 4 ) trytime = atoi( argv[4] );
  //rytime =1;

  nodeinfo node_void;
  node_void.make_void();   // strange,  one way to mark one special void
  // may be there are better ways to communicate on this topi
  // think!

  hash<nodeinfo>  node_hash( NODEHASH , node_void);  // = 16384, only 9864 needed
  //hash<nodeinfo>  node_hash( 15, node_void);  // = 16384, only 9864 needed

  linkinfo link_void;
  link_void.make_void();
  hash<linkinfo>  link_hash( LINKHASH , link_void);

  if (level0)
      Level0Prepare();
  ReadNodeFile( node_hash );
  ReadLinkFile( node_hash, link_hash );
  ReadFeedback( node_hash, link_hash );
  if (level0)
      Level0ComputeCorrections(node_hash);
  OpenPlanFile();

//    cout << scale << "   "  << maxspeed << endl;
//   cout << "-------------" << endl;

  int skipLink, oldPlan=-2;
  //  int planNo=-1, mode, vehicleID, driverPlanNo,startAcc,nSteps,startTime1;
  //  int link1,lastLink,endAcc;

  Xmin_dist = new float[arraysize];
  Xreached_by_nw = new hashindex[arraysize];
  Xreached_by_link= new hashindex[arraysize];
  Xreached_by_nfa=new int[arraysize];
  generation= new int[arraysize];

  int i;
  for( i=0; i<arraysize; i++) generation[i]=-2;


  linkinfo startinfo,destinfo;

  hashindex *outlinks = new hashindex[MAXLENGTH];  
  float     *outdists  = new float[MAXLENGTH];


  outdists[0] = 0.0;
//   dijkstra(&node_hash.retrieve (1566), &node_hash.retrieve(4439), 
// 		       outnodes, outdists, node_hash );

  int plan,dumtime;
  double update_prob;
  float xtime;
  for( plan=0 ; plan < numb_plans ; plan++ )
    {
	TRoute Route;

      int age;

      //      printf("x%dx", plan );
      if( plan % 100 == 0 ) printf("#"),fflush(stdout);

      if (!ReadRoute(planFP, &Route))
      {
	  fprintf(stderr, "\napparently end of file\n");
	  break;
      }

      // <<<< MaRi
      RoutesInFile++;
      // >>>>

      // mode is used as iteration of last replanning

      if( update_age && iteration == 1)
	Route.mode = 0;

      age = iteration - Route.mode;
      if (Route.mode == 0)
	age += initial_age;

      if (reduce_frac == 0.0)
	{
	  // the 'normal' case: treat all routes equally or according to their age

	  if (linear_age_dependant)
	    update_prob = adj_par * (age + 1);
	  else
	    update_prob = repl_frac;
	}
      else
	{
	  // the 'new' case: treat plans that have an age stamp of '0' special

	  if (Route.mode == 0)
	    update_prob = reduce_frac;
	  else
	    if (linear_age_dependant)
	      update_prob = adj_par * age;
	    else
	      update_prob = repl_frac;
	}
      if( drand48() < update_prob )
	{
	    int link1, startTime1, lastLink, endAcc;

	  // <<<< MaRi
	  RoutesReplanned++;
	  // >>>>
	  if (update_age)
	    Route.mode = iteration;
	  if (RoutesReplanned % 10 == 0)
	      printf("+"),fflush(stdout);
	  /*
	  for(skipLink = 0; skipLink < (nSteps-1); skipLink++)
	    {
		fscanf(planFP, "%d %d", &lastLink, &endAcc);
	    }	    
	  fscanf(planFP, "%d %d",  &endAcc, &dumtime);
	  */

	  link1 = Route.Entries[0].ID;
	  startTime1 = Route.Entries[0].Estimate;
	  lastLink = Route.Entries[Route.steps - 1].ID;
	  endAcc = Route.Entries[Route.steps].ID;

      //      if ( plan < numb_skip ) continue;
	  
	  hashindex startl= link_hash.index(link1);
	  hashindex destl = link_hash.index(lastLink);
	  if ( startl == NoHashIndex  ) 
	    {
	      printf(" bad start link in plan %d: %d\n", Route.planNo, link1);
	      exit(1);
	    }
	  if ( destl == NoHashIndex )
	    {
	      printf(" bad dest link %d in plan %d\n", lastLink, Route.planNo);
	      exit(2);
	    }
	  hashindex starti = link_hash[startl].NodeA;
	  hashindex desti =  link_hash[destl ].NodeB;
	  
//       start = node_hash[starti].id;
//       dest  = node_hash[desti].id;
	  int steps = dijkstra( startTime1-18000, startl, destl, outlinks, outdists, 
				node_hash, link_hash,"c", "cmp" );

	  if ( 0 == steps ) printf("Zero number of links in plan %d\n", Route.planNo );

	  Route.steps = steps + 2;
	  Route.Entries[steps + 2].ID = endAcc;
	  Route.Entries[steps + 2].Estimate = (long) (startTime1+outdists[0]);
	  Route.Entries[steps + 1].ID = lastLink;
	  Route.Entries[steps + 1].Estimate = (long) (startTime1+outdists[0]);
	  while( steps >= 0 ) 
	  {
	      Route.Entries[Route.steps - steps - 2].ID = link_hash[outlinks[steps]].id;
	      Route.Entries[Route.steps - steps - 2].Estimate = (long) (startTime1 + outdists[steps]);
	      steps--;
	  }
	}
      else // don't replan
	{
	  int oldTime;	  
	  int CStep;
	  int link1, startTime1, lastLink;

	  //	  fprintf( outFP, "%d %d %d %d %d %d %d %d", planNo, mode, vehicleID, driverPlanNo,
	  //		 startAcc, nSteps, link1, startTime1);
	      if( update_estimates )
		{   // find starting node and time
		    
		    //		    fprintf(stderr, "cannot update estimate!\n");
		    //		    exit (3);
		    hashindex nextlinki,linkindex,curnodei,oldnodei;
		    int time;

		    link1 = Route.Entries[0].ID;
		
		    //		    fscanf(planFP, "%d%d", &lastLink, &oldTime);

		    CStep = 1;
		    lastLink = Route.Entries[CStep].ID;
		    oldTime = Route.Entries[CStep].Estimate;

		    linkindex=link_hash.index(link1);
		    nextlinki=link_hash.index(lastLink);
		  
		    curnodei = link_hash[linkindex].NodeA;
		    nodeinfo &curnode = node_hash[curnodei];
		    for( i = 0; -1 ; i++ )
		    {
			if( i >= curnode.Xdegree[0] ) 
			{
			  //			  printf("planB\n");
			  curnodei = link_hash[linkindex].NodeB;
			  nodeinfo &curnodey = node_hash[curnodei];
			  for( i = 0; -1 ; i++ )
			    {
			      if( i >= curnodey.Xdegree[0] ) 
				{
				    printf("inconsistent (first edge) plan %d\n", Route.planNo); 
				    exit(16);
				}
			      if( nextlinki == curnodey.Xlinks[0][i] ) 
				{
				  time = startTime1 + 0.5*TrTime( curnodei, i , startTime1 - 18000);
				  //  curnodei=link_hash[linkindex].NodeA;
				  break;
				}
			    }
			  break;
			}
		      if( nextlinki == curnode.Xlinks[0][i] ) 
			{
			  time = startTime1 + 0.5*TrTime( curnodei, i , startTime1 - 18000);
			  //			  curnodei=link_hash[linkindex].Node;
			  break;
			};
		    }
		  oldnodei=curnodei;
		  //		  fprintf( outFP, " %d %d", lastLink, time );
		  
		  Route.Entries[CStep].Estimate = time;

		  for(skipLink = 0; skipLink < Route.steps-2; skipLink++)
		    {
			CStep++;
		      linkindex = link_hash.index(lastLink);
		      //		      printf("l:%d %d\n", linkindex, nextlinki);
		      int oldLink=lastLink;
		      //		      fscanf(planFP, "%d%d", &lastLink, &oldTime);

		      lastLink = Route.Entries[CStep].ID;
		      
		      if( skipLink % 5 == 0 ) fprintf(outFP, "\n");
		      nodeinfo &curnodex = node_hash[curnodei];
		      for( i = 0; -1 ; i++ )
			{
			  if( i >= curnodex.Xdegree[0] ) 
			    {
			      printf("inconsistent plan %d\n", Route.planNo); exit(17);
			    }
			  if( linkindex == curnodex.Xlinks[0][i] ) 
			    {
			      break;
			    }
			}
		      xtime =  TrTime( curnodei, i , time -18000);
		      if ( xtime == INFTY )
			{
			  xtime=node_hash[curnodei].Xdistances[0][i];
			  printf("xx %d (%d) :%f\n", oldLink,time-18000,xtime ); 
			}
		      time = time + xtime;
		      //		      fprintf(outFP, " %d %d ", lastLink, time );

		      Route.Entries[CStep].Estimate = time;

		      oldnodei = curnodei;
		      curnodei = node_hash[curnodei].Xneighbours[0][i];
		    }
		  //		  fscanf(planFP, "%d%d", &lastLink, &oldTime);

		  CStep++;
		  lastLink = Route.Entries[CStep].ID;

		  xtime =  TrTime( oldnodei, i , time - 18000);
		  if ( xtime == INFTY )
		    {
		      xtime=node_hash[oldnodei].Xdistances[0][i];
		      printf("yy {%d} %d [%d] (%d) :%f\n", Route.planNo,
			     lastLink,node_hash[oldnodei].Xlinks[0][i],time-18000,xtime ); 
		    }
		  time = time + 0.5*xtime;
		  //		  time = time + .5*TrTime( curnodei, i , time );
		  //		  fprintf(outFP, " %d %d ", lastLink, time );

		  Route.Entries[CStep].Estimate = time;

		}
	}
      WriteRoute(outFP, &Route);
      if (pipe_name)
	if (RoutesInFile % 10 == 0)
	{
	    fflush(outFP);
	    SyncStep(&SyncInfo, RoutesInFile - 1);
	}
#ifdef printroutes
      pr_coords();
      increase( countc );
#endif
    }

  if (pipe_name)
  {
      fclose(outFP);
      SyncStep(&SyncInfo, RoutesInFile);
  }
  // <<<< MaRi
  printf("\nactual replanning fraction (%ld out of %ld): %f\n", 
	 RoutesReplanned, RoutesInFile, 1.0 * RoutesReplanned / RoutesInFile);
  // >>>>
  delete Xmin_dist;
  delete Xreached_by_nw;
  delete Xreached_by_link;
  delete Xreached_by_nfa;
  delete generation;
  
  delete outlinks;
  delete outdists;

  if (pipe_name)
  {
      long Step = 1;
      int  Error;

      SyncStep(&SyncInfo, 12345678);
      fprintf(stderr, "waiting for converter to complete...\n");
      while (SyncWait(&SyncInfo, Step, Error))
	  sleep(1);
      if (Error)
	  fprintf(stderr, "error while waiting for converter to complete!\n");
      else
	  fprintf(stderr, "converter completed successfully!\n");
      SyncDeactivate(&SyncInfo);
  }
  return 0;
}







