/*      file     : Planner/hah.cc
        purpose  : hashing
        author   : Riko Jacob
        RCS      : $Id: hash.cc,v 1.3 2005/03/25 12:27:31 mr Exp $

        Copyright (C) 2005 Riko Jacob
*/

// Only needed because of non-consecutive node / link numbers
// hash function designed for this:
// "Semantics" in Numbers start at plain thousands or so, 
// because of that something like "bit-and 2^i - 1" should be apropriate

// finite chains: k*h(x) + linear probing  should work
// if k is not large enough things will get real bad

// one could als recommend that the substituted clas INFO somehow provides 
// the hashing function. But let's stay with easy things


// try pseudo linear probing

#define linprobe 1

// this is to keep dijkstra as clean as possible. Only access and adress-of operations
// depend on it.
// Hopefully Compiler optimization can lead us back to pointers without loss of efficiency
// may be care should be taken if links and nodes want to be treated differently
typedef unsigned short int hashindex;
#define NoHashIndex ((hashindex)-1)

template<class INFO>
class hash
{
public:
  hash(int s,INFO hav ) :  ins_collisions(0), retrieve_collisions(0), max_collisions(0),
    size( 1 << s), ll(s), 
    hashmask(size-1), 
    hashvoid(hav)
  {
    int i;
    table = new INFO[size]; 
    for(i=0;i<size;i++) table[i].id=-1;
    //    cout << int(table) << "   " << size <<":"<< hashmask<<endl;
  };
  ~hash() 
  { 
    delete table; 
//     cout << "Hash table collisions (ins/retr/max): (" 
// 	 << ins_collisions <<"/"<<retrieve_collisions<<"/"<<max_collisions <<")"<<endl;
  };

  hashindex insert( INFO info);
  INFO  &retrieve( const int label ); 
  hashindex index( const int label );
  INFO  &operator[](const hashindex index);

  unsigned long ins_collisions;
  unsigned long retrieve_collisions;
  int max_collisions;

private:
  int   scramble( int x) 
  {
    return ( (3*x) + ((x*x>>ll)) ) & hashmask; 
  };

  INFO *table;
  const int size,ll;
  const int hashmask;
  const INFO hashvoid;    // this is the constant standing for non-members
  INFO  hv;               // this will be returned and can be changed from calling proc.
};


template<class INFO>
hashindex hash<INFO>::insert( INFO info)
{
  int i,start, coll=0;   // run variable for probing
  //  for(i=0;i<size;i++) cout << labels[i] <<":"; cout <<  endl;
  if ( info.id == -1 ) exit(25);  //     -1 is not a valid label    (used as marker for unused)

  i = start = scramble(info.id);
  while( table[i].id != -1 )  
    {
      //      cout << i << endl;
      if ( table[i].id == info.id ) exit(27); // two items with same label inserted
      i=hashmask&(i+linprobe);
      if ( i == start ) exit(26); // hash table overflow
      coll++;
      //  cout << "hash:collision"<<endl;
    }
  table[i] = info;
  ins_collisions += coll;
  if ( coll > max_collisions ) max_collisions=coll;
  //  for(i=0;i<size;i++) cout << table[i] <<"*";cout<<endl;
  return i;  
}  

template<class INFO>
INFO  &hash<INFO>::retrieve( const int label )
{
  hashindex ind = index( label );
  if ( ind == NoHashIndex ) 
    {
      hv = hashvoid;
      return hv;
    }
  return operator[](ind);  
}

template<class INFO>
hashindex hash<INFO>::index(const int label)
{
  int i,start,coll=0;   // run variable for probing

  i = start= scramble(label);

  while( (table[i].id != label) && (table[i].id != -1) ) 
    {
      coll++;
      i=hashmask&(i+linprobe);
      if ( i==start ) return NoHashIndex;
    }
  retrieve_collisions += coll;
  if ( coll > max_collisions ) max_collisions=coll;  

  if ( table[i].id == -1 ) 
    return NoHashIndex;
  else
    return (hashindex) i;
}

template<class INFO>
inline INFO &hash<INFO>::operator[](const hashindex index)
{
  return table[index];
}
