/*      file     : TrafficData/TGridStatLookup.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TGridStatLookup.C,v 1.5 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TClassInfo.h>
#include "TEdgeInfo.h"
#include "TGridStatLookup.h"

// ***DEFINES************************************************************************* 
// ***CONSTANTS***********************************************************************
// ***TYPES*************************************************************************** 
// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *GridStatLookupClassInfo = 0;
TClassInfo *LocalGridStatLookupClassInfo = 0;

// ***LOCAL VARIABLES***************************************************************** 
// ***FUNCTIONS*********************************************************************** 
// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 
#define ParentClass TTimeLookup
// =================================================================================== 

TGridStatLookup::TGridStatLookup(TClassInfo *theClassInfo, TParallelID theParallelID) :

ParentClass(theClassInfo, theParallelID)

{    
}

// =================================================================================== 

TGridStatLookup::~TGridStatLookup(void)

{
}

// =================================================================================== 

double
TGridStatLookup::GetAverageDensity(long TimeStep)

{
#ifdef USE_LATEST_GRID_STAT
    TStatObject *Latest;

    if (!GetLatest(Latest))
    {
        Warning("GetAverageDensity: cannot retrieve stats");
        return (double) INVALID;
    }
    return ((TGridStat *) Latest)->GetDensity();
#else
    TStatObject *First, *Second;
    double       FirstWeight, SecondWeight;

    if (!Get(TimeStep, First, FirstWeight, Second, SecondWeight))
    {
        Warning("GetAverageDensity: cannot retrieve stats");
        return (double) INVALID;
    }
    return (FirstWeight * ((TGridStat *) First)->GetDensity() + 
            SecondWeight * ((TGridStat *) Second)->GetDensity());
#endif
}

// =================================================================================== 

double
TGridStatLookup::GetAverageTravelTime(long TimeStep)

{
#ifdef USE_LATEST_GRID_STAT
    TStatObject *Latest;
    long         Dummy;
    double       TravelTime;

    if (!GetLatest(Latest))
    {
        Warning("GetAverageTravelTime: cannot retrieve stats");
        return (double) INVALID;
    }
    if (!((TGridStat *) Latest)->GetConsistentTravelTime(Dummy, TravelTime))
        return INVALID;
    return TravelTime;
#else
    TStatObject *First, *Second;
    long         Dummy;
    double       FirstWeight, SecondWeight;    
    double       FirstTravelTime, SecondTravelTime;

    if (!Get(TimeStep, First, FirstWeight, Second, SecondWeight))
    {
        Warning("GetAverageTravelTime: cannot retrieve stats");
        return (double) INVALID;
    }
    if (!((TGridStat *) First)->GetConsistentTravelTime(Dummy, FirstTravelTime))
        return INVALID;
    if (!((TGridStat *) Second)->GetConsistentTravelTime(Dummy, SecondTravelTime))
        return INVALID;
    return (FirstWeight *  FirstTravelTime + SecondWeight * SecondTravelTime);
#endif
}

// =================================================================================== 

double
TGridStatLookup::GetAverageVelocity(long TimeStep)

{
#ifdef USE_LATEST_GRID_STAT
    TStatObject *Latest;
    double       VMean;

    if (!GetLatest(Latest))
    {
        Warning("GetAverageVelocity: cannot retrieve stats");
        return (double) INVALID;
    }
    if (!((TGridStat *) Latest)->GetConsistentVMean(VMean))
        return INVALID;
    return VMean;
#else
    TStatObject *First, *Second;
    double       FirstWeight, SecondWeight;
    double       FirstVMean, SecondVMean;

    if (!Get(TimeStep, First, FirstWeight, Second, SecondWeight))
    {
        Warning("GetAverageVelocity: cannot retrieve stats");
        return (double) INVALID;
    }
    if (!(((TGridStat *) First)->GetConsistentVMean(FirstVMean) AND
          ((TGridStat *) First)->GetConsistentVMean(SecondVMean)))
        return INVALID;
    return (FirstWeight * FirstVMean + SecondWeight * SecondVMean);
#endif
}

// =================================================================================== 
#undef ParentClass
