/*      file     : TrafficData/TMultilaneDump.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TMultilaneDump.C,v 1.4 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <TBaseStream.h>
#include <TMultilaneGrid.h>

#include "TRouteplan.h"
#include "TMultilaneDump.h"

// ***DEFINES************************************************************************* 
// ***CONSTANTS***********************************************************************
// ***TYPES*************************************************************************** 
// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TMultilaneDump::ClassInfo = 0;

// ***LOCAL VARIABLES***************************************************************** 
// ***FUNCTIONS*********************************************************************** 
// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TStatObject
// =================================================================================== 

TMultilaneDump::TMultilaneDump(TParallelID theParallelID) :

ParentClass(ClassInfo, theParallelID)

{
    Reset();
}

// =================================================================================== 

TMultilaneDump::~TMultilaneDump(void)

{
    int CLane;

    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (DumpEntries[CLane])
            free(DumpEntries[CLane]);
}

// =================================================================================== 

bool
TMultilaneDump::Accumulate(const TStatObject *StatObject)

{
    TMultilaneDump *OtherDump = (TMultilaneDump *) StatObject;
    int  CLane;

    if (NrOfLanes != OtherDump->NrOfLanes)
    {
        Error("Accumulate: numbers of lanes do not match");
        return FALSE;
    }
    if (FirstSite != OtherDump->LastSite AND
        LastSite != OtherDump->FirstSite)
    {
        Error("Accumulate: ranges do not match");
        return FALSE;
    }
    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (OtherDump->NrOfEntries[CLane] > 0)
        {
            long NewCount = NrOfEntries[CLane] + OtherDump->NrOfEntries[CLane];

            if ((DumpEntries[CLane] = 
                 (TDumpEntry *) realloc(DumpEntries[CLane], 
                                        NewCount * sizeof(TDumpEntry))) == 0)
            {
                Error("Accumulate: out of memmory");
                return FALSE;
            }
            memcpy((void *) (DumpEntries[CLane] + NrOfEntries[CLane]),
                   (void *) OtherDump->DumpEntries[CLane], 
                   OtherDump->NrOfEntries[CLane] * sizeof(TDumpEntry));
            NrOfEntries[CLane] = NewCount;
        }
    return ParentClass::Accumulate(StatObject);
}

// =================================================================================== 

bool
TMultilaneDump::Allocate(void)

{
    int CLane;

    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (DumpEntries[CLane] == 0 AND NrOfEntries[CLane] > 0)
            if ((DumpEntries[CLane] = 
                 (TDumpEntry *) malloc(NrOfEntries[CLane] * sizeof(TDumpEntry))) == 0)
            {
                Error("Allocate: out of memory");
                return FALSE;
            }
    return TRUE;
}

// =================================================================================== 

bool
TMultilaneDump::Collect(const TMultilaneGrid *Grid)

{
    int CLane;
    long CSite;

    if (NrOfLanes != 0)
    {
        Error("Collect: trying to collect again");
        return FALSE;
    }
    NrOfLanes = Grid->GetNrOfLanes();
    FirstSite = Grid->GetFirstSite();
    LastSite = Grid->GetLastSite();
    for (CLane = 0; CLane < NrOfLanes; CLane++)
        for (CSite = FirstSite; CSite < LastSite; CSite++)
            if (Grid->SAccess(CLane, CSite) & SITE_DATA_MASK)
                NrOfEntries[CLane]++;
    if (!Allocate())
    {
        Error("Collect: cannot allocate");
        return FALSE;
    }
    for (CLane = 0; CLane < NrOfLanes; CLane++)
    {
        long Count = 0;
        
        for (CSite = FirstSite; CSite < LastSite; CSite++)
            if (Grid->SAccess(CLane, CSite) & SITE_DATA_MASK)
            {
                TVehicle   *Vehicle;
                TRouteplan *Routeplan = 0;
                float       RelativeDelay = INVALID;

                DumpEntries[CLane][Count].Position = CSite;
                DumpEntries[CLane][Count].Site = Grid->SAccess(CLane, CSite);

                if ((Vehicle = Grid->VAccess(CLane, CSite)) AND
                    (Routeplan = Vehicle->GetRouteplan()) AND
                    (RelativeDelay = Routeplan->GetEstimatedRelativeDelay()) != INVALID)
                {
                    int DelayCode;

                    //              Print("Start:%ld TimeStep %s, EstimatedRemTT: %ld, "
                    //  "Insertion:%ld, EstimatedTT:%ld", StartTimeStep,
                    //  (char *) GetTimeStep().GetInstanceName(), 
                    //  Routeplan->GetEstimatedRemainingTravelTime(),
                    //  Routeplan->GetInsertionTimeStep(), Routeplan->GetEstimatedTravelTime());

                    if (RelativeDelay < -0.5)
                        DelayCode = 1;
                    else if (RelativeDelay < -0.2)
                        DelayCode = 2;
                    else if (RelativeDelay < -0.1)
                        DelayCode = 3;
                    else if (RelativeDelay < 0.1)
                        DelayCode = 4;
                    else if (RelativeDelay < 0.2)
                        DelayCode = 5;
                    else if (RelativeDelay < 0.5)
                        DelayCode = 6;
                    else
                        DelayCode = 7;
                    SITE_SET_DELAY(DumpEntries[CLane][Count].Site, DelayCode);
                    if (Routeplan->GetNrOfReroutes() > 0)
                        DumpEntries[CLane][Count].Site |= SITE_REROUTED_VEHICLE;
                }
                Count++;
            }
    }
    return TRUE;
}

// =================================================================================== 

bool
TMultilaneDump::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    int CLane;

    if (!(ParentClass::Decode(aStream, TransferInfo) AND
          aStream->Decode(NrOfLanes, TMULTILANEDUMP_LONG)))
        return FALSE;
    if (!Allocate())
    {
        Error("Decode: cannot allocate");
        return FALSE;
    }
    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (!aStream->Decode(DumpEntries[CLane][0].Position, 2 * NrOfEntries[CLane]))
            return FALSE;
    return TRUE;
}

// =================================================================================== 

bool
TMultilaneDump::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    int CLane;

    if (!(ParentClass::Encode(aStream, TransferInfo) AND
          aStream->Encode(NrOfLanes, TMULTILANEDUMP_LONG)))
        return FALSE;
    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (!aStream->Encode(DumpEntries[CLane][0].Position, 2 * NrOfEntries[CLane]))
            return FALSE;
    return TRUE;
}

// =================================================================================== 

TString
TMultilaneDump::GetClassName(void) const

{
    return "TMultilaneDump";
}

// =================================================================================== 

TString
TMultilaneDump::GetInstanceName(void) const

{
    char Line[256];

    sprintf(Line, "(%s) Lanes:%ld Range [%ld:%ld]", 
            (char *) ParentClass::GetInstanceName(), NrOfLanes, FirstSite, LastSite - 1);
    return Line;
}

// =================================================================================== 

void
TMultilaneDump::Reset(void)

{
    int CLane;

    NrOfLanes = 0;
    for (CLane = 0; CLane < MAX_LANES; CLane++)
    {
        NrOfEntries[CLane] = 0;
        DumpEntries[CLane] = 0;
    }
}

// =================================================================================== 
#undef ParentClass
