/*      file     : TrafficData/TMultilaneDump.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TMultilaneDump.h,v 1.3 2005/03/25 12:28:51 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMULTILANEDUMP_H
#define TMULTILANEDUMP_H

// ***INCLUDES************************************************************************ 

#include <CAConfig.h>
#include <TStatObject.h>

// ***FORWARDS************************************************************************

class TMultilaneGrid;

// ***CLASSES************************************************************************* 

struct TDumpEntry 

{
    long Position;
    long Site;
};

// ===================================================================================

class TMultilaneDump : public TStatObject

{
public:

    TMultilaneDump(TParallelID theParallelID);
    virtual ~TMultilaneDump(void);

    bool
    Collect(const TMultilaneGrid *Grid);

    inline TDumpEntry &
    GetEntry(int Lane, int Entry);

    inline long
    GetNrOfEntries(int Lane) const;

    inline long
    GetNrOfLanes(void) const;

    static TClassInfo *ClassInfo;

private:

    virtual bool
    Accumulate(const TStatObject *StatObject);

    bool
    Allocate(void);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const;

    virtual TString
    GetClassName(void) const;

    virtual TString
    GetInstanceName(void) const;

    virtual void
    Reset(void);

    //  >>> DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define TMULTILANEDUMP_LONG (3 + MAX_LANES)
    long         NrOfLanes;
    long         FirstSite;
    long         LastSite;
    long         NrOfEntries[MAX_LANES];
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

    TDumpEntry  *DumpEntries[MAX_LANES];
};

// ***TYPES*************************************************************************** 

typedef TDLList < TMultilaneDump > TMultilaneDumpList;
typedef TDLListEnum < TMultilaneDump > TMultilaneDumpEnum;

// ***INLINE METHODS******************************************************************

#ifndef NO_INLINE
#  include "TMultilaneDump.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
