/*      file     : TrafficData/TMultilaneGridConfig.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TMultilaneGridConfig.C,v 1.6 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TBaseStream.h>

#include "TMultilaneGridConfig.h"

// ***METHODS************************************************************************* 

TMultilaneGridConfig::TMultilaneGridConfig(void)

{
    int CLane;

    NrOfSites = INVALID;
    ChangeViewRange = CA_DEFAULT_CHANGE_VIEW_RANGE;
    ChangeUpdateSteps = CA_DEFAULT_CHANGE_UPDATE_STEPS;
    MoveViewRange = CA_DEFAULT_MOVE_VIEW_RANGE;
    MoveUpdateSteps = CA_DEFAULT_MOVE_UPDATE_STEPS;
    MaxV = CA_DEFAULT_V_MAX;
    SpeedLimit = CA_DEFAULT_SPEED_LIMIT;
    SlowVelocity = 3;
    NrOfLanes   = CA_DEFAULT_NR_OF_LANES;
    DecProb     = CA_DEFAULT_DEC_PROB;
    LookBack[CHANGE_LEFT] = CA_DEFAULT_LEFT_LOOKBACK;
    LookBack[CHANGE_RIGHT] = CA_DEFAULT_RIGHT_LOOKBACK;
    RouteLookBack[CHANGE_LEFT] = CA_DEFAULT_ROUTE_LEFT_LOOKBACK;
    RouteLookBack[CHANGE_RIGHT] = CA_DEFAULT_ROUTE_RIGHT_LOOKBACK;
    ChangeProb[CHANGE_LEFT] = CA_DEFAULT_RL_PROB;
    ChangeProb[CHANGE_RIGHT] = CA_DEFAULT_LR_PROB;
    Flags = CA_DEFAULT_FLAGS;
    for (CLane = 0; CLane < MAX_LANES; CLane++)
        LaneFlags[CLane] = 0;
}

/* =================================================================================== */

bool
TMultilaneGridConfig::Decode(TBaseStream *aStream)

{
    long CLane;

    if (!(aStream->Decode(Flags) AND
          aStream->Decode(NrOfSites) AND
          aStream->Decode(NrOfLanes) AND
          aStream->Decode(MaxV) AND
          aStream->Decode(SpeedLimit) AND
          aStream->Decode(MoveUpdateSteps) AND
          aStream->Decode(MoveViewRange) AND
          aStream->Decode(ChangeUpdateSteps) AND
          aStream->Decode(ChangeViewRange) AND
          aStream->Decode(DecProb) AND
          aStream->Decode(LookBack[CHANGE_LEFT]) AND
          aStream->Decode(LookBack[CHANGE_RIGHT]) AND
          aStream->Decode(ChangeProb[CHANGE_LEFT]) AND
          aStream->Decode(ChangeProb[CHANGE_RIGHT])))
        return FALSE;
    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (!aStream->Decode(LaneFlags[CLane]))
            return FALSE;
    return TRUE;
}

/* =================================================================================== */

bool
TMultilaneGridConfig::Encode(TBaseStream *aStream) const

{
    long CLane;

    // Trace("Encode");

    if (!(aStream->Encode(Flags) AND
          aStream->Encode(NrOfSites) AND
          aStream->Encode(NrOfLanes) AND
          aStream->Encode(MaxV) AND
          aStream->Encode(SpeedLimit) AND
          aStream->Encode(MoveUpdateSteps) AND
          aStream->Encode(MoveViewRange) AND
          aStream->Encode(ChangeUpdateSteps) AND
          aStream->Encode(ChangeViewRange) AND
          aStream->Encode(DecProb) AND
          aStream->Encode(LookBack[CHANGE_LEFT]) AND
          aStream->Encode(LookBack[CHANGE_RIGHT]) AND
          aStream->Encode(ChangeProb[CHANGE_LEFT]) AND
          aStream->Encode(ChangeProb[CHANGE_RIGHT])))
        return FALSE;
    for (CLane = 0; CLane < NrOfLanes; CLane++)
        if (!aStream->Encode(LaneFlags[CLane]))
            return FALSE;
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
