/*      file     : TrafficData/TRoutingStat.h
        project  : PAMINA III
        authors  : Marcus Rickert
        RCS      : $Id: TRoutingStat.h,v 1.4 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TROUTINGSTAT_H
#define TROUTINGSTAT_H

// ***INCLUDES************************************************************************ 

#include <TStatObject.h>

// ***FORWARDS************************************************************************ 

class TTimeStep;
class TSource;
class TSink;

// ***CLASSES************************************************************************* 

class TRoutingStat : public TStatObject

{
public:

    TRoutingStat(TParallelID theParallelID = INVALID);

    virtual ~TRoutingStat(void);

    inline void
    AddVehicles(long theNrOfVehicles);

    inline void
    Collect(const TSource *Source);

    inline void
    Collect(const TSink *Sink);

    inline void
    CountRouteplanFile(void);

    inline long
    GetDeletions(void) const;

    inline long
    GetDelayedInsertions(void) const;

    inline long
    GetNormalInsertions(void) const;

    inline long
    GetInsertions(void) const;

    inline long
    GetNrOfRoutedVehicles(void) const;

    inline long
    GetNrOfSources(void) const;

    inline long
    GetNrOfSinks(void) const;

    inline long
    GetNrOfVehicles(void) const;

    inline long
    GetOpenRouteplanFiles(void) const;

    inline long
    GetPendingVehicles(void) const;

    inline double
    GetRatioOfRoutedVehicles(void) const;

    inline long
    GetRoutesFailed(void) const;

    virtual void
    PrintScreenStat(FILE *OutputFile);

    bool
    Write(const TTimeStep &, TStream *Stream);

    static bool
    WriteStatDescription(TStream *Stream);

    static TClassInfo *ClassInfo;

private:
 
    virtual bool
    Accumulate(const TStatObject *StatObject);

    virtual bool
    Decode(TBaseStream *aStream, TTransferInfo *TransferInfo);

    virtual bool
    Encode(TBaseStream *aMessage, TTransferInfo *TransferInfo) const;

    virtual void
    Reset(void);

    //  >>> DO NOT SEPERATE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
#   define ROUTING_STAT_LONG 10
    long NrOfSources;
    long NrOfSinks;
    long NrOfVehicles;
    long NormalInsertions;
    long DelayedInsertions;
    long Deletions;
    long PendingVehicles;
    long RoutesFailed;
    long OpenRouteplanFiles;
    long SumDelay;
    //  >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
};

// ***INLINE METHODS****************************************************************** 

#ifndef NO_INLINE
#  include "TRoutingStat.i.C"
#endif

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
