/*      file     : TrafficData/TSink.C
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TSink.C,v 1.4 2005/03/27 20:39:16 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TCommunication.h>
#include <TObjectManager.h>

#include <TTimeStep.h>
#include <TBaseStream.h>
#include <TTransferInfo.h>

#include <TrafficDataTypes.h>

#include <TVehicle.h>
#include <TRouteplan.h>
#include <TRouter.h>
#include <TMultilaneGrid.h>

#include "TSink.h"

// ***GLOBAL VARIABLES****************************************************************  

TClassInfo *TSink::ClassInfo = 0;

// ***METHODS************************************************************************* 

TSinkVarData::TSinkVarData(void)

{
    Deletions = 0;
    RoutesFailed = 0;
}

// =================================================================================== 
#define ParentClass TParallelObject
// =================================================================================== 

TSink::TSink(TParallelID theParallelID) : 

ParentClass(ClassInfo, theParallelID)

{
    Grid = 0;
    Offset = INVALID;
}

/* =================================================================================== */

bool
TSink::AssignGrid(TMultilaneGrid *theGrid, long theOffset)

{
    Grid = theGrid;
    Offset = theOffset;
    return TRUE;
}

/* =================================================================================== */

bool
TSink::DestroyVehicle(const TTimeStep &TimeStep, TVehicle *&Vehicle)

{
    TRouteplan   *Routeplan;

    Print(DM_ROUTE,"vehicle %s destroyed", (char *) Vehicle->GetInstanceName());
    if ((Routeplan = Vehicle->GetRouteplan()))
    {
        if (!Router->HandleExecutedRouteplan(TimeStep, Routeplan))
        {
            Error("DestroyVehicle: while passing routeplan to TRouter");
            return FALSE;
        }
        Vehicle->SetRouteplan(0);
    }
    DeleteObject(Vehicle);
    Var.Deletions++;
    return TRUE;
}

/* =================================================================================== */

bool
TSink::Decode(TBaseStream *aStream, TTransferInfo *TransferInfo)

{
    if (!ParentClass::Decode(aStream, TransferInfo))
        return FALSE;
    switch(TransferInfo->TransferMode)
    {
    case TM_ConstData:
        return TRUE;

    case TM_VarData:
        return aStream->Decode(Var.Deletions, TSINK_LONG);

    default:
        Warning("Decode: invalid TransferMode %d", TransferInfo->TransferMode);
    }
    return TRUE;
}

// =================================================================================== 

bool
TSink::Encode(TBaseStream *aStream, TTransferInfo *TransferInfo) const

{
    if (!ParentClass::Encode(aStream, TransferInfo))
        return FALSE;
    switch(TransferInfo->TransferMode)
    {
    case TM_ConstData:
        return TRUE;
        
    case TM_VarData:
        return aStream->Encode(Var.Deletions, TSINK_LONG);

    default:
        Warning("Encode: invalid TransferMode %d", TransferInfo->TransferMode);
    }
    return TRUE;
}

// =================================================================================== 

bool
TSink::ExecuteSubTimeStep(const TTimeStep &TimeStep)

{
    TVehicle    *Vehicle;
    TRouteplan  *Routeplan;
    long         i, CLane;

    if (Grid AND Grid->VehicleRangeIsActive(Offset, CA_DEFAULT_V_MAX))
        for (CLane = 0; CLane < Grid->GetNrOfLanes(); CLane++)
            for (i = 0; i < CA_DEFAULT_V_MAX; i++)
                if ((Vehicle = Grid->VAccess(CLane, Offset + i)))
                    if ((Routeplan = Vehicle->GetRouteplan()) AND
                        Routeplan->IsLastAccessory(GetParallelID()))
                    {
                        Grid->RemoveVehicle(CLane, Offset + i, FALSE);
                        DestroyVehicle(TimeStep, Vehicle);
                    }
    return TRUE;
}

// =================================================================================== 

TString
TSink::GetClassName(void) const

{
    return "TSink";
}

// =================================================================================== 

void
TSink::VehicleFailed(TVehicle *Vehicle)

{
    Warning("vehicle %s removed", (char *) Vehicle->GetInstanceName());
    Var.RoutesFailed++;
    Var.Deletions++;
    DeleteObject(Vehicle);
}

// =================================================================================== 
#undef ParentClass
