/*      file     : TrafficData/TTrafficSimulator.h
        project  : PAMINA III
        author   : Marcus Rickert
        RCS      : $Id: TTrafficSimulator.h,v 1.10 2005/03/25 12:28:52 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TTRAFFICSIMULATOR_H
#define TTRAFFICSIMULATOR_H

// ***INCLUDES************************************************************************ 

#include <TBaseObject.h>
#include <TObjectHandler.h>
#include <TStatHandler.h>

// ***FORWARDS************************************************************************

class TCAManager;
class TTimer;

#include <TNode.forward.h>
#include <TEdge.forward.h>

// ***CLASSES************************************************************************* 

class TTrafficSimulator : public TBaseObject, public TObjectHandler, public TStatHandler

{
public:

    TTrafficSimulator(TNodeList *theNodes, TEdgeList *theEdges, 
                      TEdgeList *theBoundaryEdges, TEdgeList *theNonBoundaryEdges,
                      TTimer *theNodeTimer, TTimer *theEdgeTimer);
    virtual ~TTrafficSimulator(void);

    bool
    Activate(void);

    bool
    ExecuteSubTimeStep(TTimeStep &TimeStep, bool BoundariesAvailable);

    bool
    InitializeEdgeAfterReception(TEdge *Edge, TDistributedRange *ActiveRange);

    bool
    InitializeNodeAfterReception(TNode *Node);

    bool
    Run(long theStartTimeStep);
    
private:

    bool
    ActivateStatistics(void);

    virtual bool
    Collect(TSamplingDescription *Description);
 
    bool
    CollectTurnCounts(const TTimeStep &TimeStep, TStatObjectList *StatList);

    bool
    ExecuteIntersections(const TTimeStep &TimeStep);

    bool
    ExecuteIntersectionControls(const TTimeStep &TimeStep);

    TString
    GetClassName(void) const;

    long
    GetStartTimeStep(void) const;

    virtual bool
    HandleAggregationResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    virtual bool
    HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                        TParallelID ParallelID);
    virtual bool
    HandleLoadRequest(TParallelObject *Object, bool &Loaded);

    bool
    HandleTurnCountResult(TAggregationInfo *AggregationInfo, TTimeStep &TimeStep);

    bool
    PrepareIntersections(const TTimeStep &TimeStep);

    long        StartTimeStep;

    TCAManager *CAManager;

    TEdgeList *BoundaryEdges;
    TEdgeList *NonBoundaryEdges;
    TEdgeList *Edges;
    TNodeList *Nodes;
    TTimer    *NodeTimer;
    TTimer    *EdgeTimer;

    TStream               *TurnCountStatOutput;
    int                    TurnCountStatInterval;
    TSamplingDescription  *TurnCountSamplingDescription;
    TAggregationInfo      *TurnCountAggregationInfo;
};

// ***EXTERNALS*********************************************************************** 

extern TTrafficSimulator *TrafficSimulator;

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif

