/*      file     : common/TAsciiDataTypeReader.h
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TAsciiDataTypeReader.h,v 1.11 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TASCIIDATATYPEREADER_H
#define TASCIIDATATYPEREADER_H

// ***INCLUDES************************************************************************ 

#include <stdio.h>

#include <TString.h>

#include <TDataTypeReader.h>

// ***DEFINES************************************************************************* 

#define MAX_LINE_LENGTH      512
#define INDEX_FILE_EXTENSION ".idx"

// ***TYPES*************************************************************************** 

// extern "C" 
struct TIndexEntry

{
    LongID Key;
    LongID Offset;
};

// ***CLASSES************************************************************************* 

/**
  */

class TAsciiDataTypeReader : public TDataTypeReader

{
public:

    TAsciiDataTypeReader(const char *theBaseFilename,
                         const char *theTableName,
                         const char *theIndexScanPattern = 0,
                         const char *theIndexExtension = INDEX_FILE_EXTENSION);
    virtual ~TAsciiDataTypeReader(void);

    void
        Close(void);
    virtual bool
        Eof(void);
    virtual void *
        Get(void) const;
    virtual void *
        Get(LongID Index);
    void *
        GetBuffered(LongID ID);
    virtual bool
        GetLinearIndex(LongID *&Index, long &Count);
    virtual void *
        GetSelected(long);
    virtual bool
        PrepareLinearAccess(void);
    virtual bool
        PrepareRandomAccess(void);
    virtual bool
        Select(LongID Index, long &Count);

protected:

    virtual void
    DisposeObject(void *);

    virtual void *
        CreateObject(const char *) const;

private:

    TString      TableName;
    TString      BaseFilename;
    TString      IndexScanPattern;
    TString      IndexFileExtension;
    FILE        *Input;
    int          IndexFile;
    TIndexEntry *Indices;
    LongID      *LinearIndex;
    long         NrOfEntries;
    long         LineNr;
    long         LineOffset;
    char         LineBuffer[MAX_LINE_LENGTH];
    bool         ReadOnly;
    long         FirstEntry;
    long         CurrentSelectionCount;
    void        *PreviousObject;
    LongID       PreviousID;
    
    bool
        CountEntries(void);
    bool
        CreateIndices(const char *Filename);
    bool
        GenerateIndices(void);
    virtual TString
        GetClassName(void) const;
    virtual TString
        GetInstanceName(void) const;
    long
        GetNrOfEntries(void);
    bool
        GetRange(LongID Key, long &First, long &Last);
    bool
        MapIndices(void);
    bool
        Open(void);
    bool
        OpenIndices(const char *Filename);
    bool
        QueryIndices(bool Create);
    bool
        ReadLine(void);
    bool
        UnmapIndices(void);
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif

