/*      file     : common/TContainer.C
        project  : Generel Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TContainer.C,v 1.5 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include "TString.h"
#include "TEnumerator.h"

#include "TContainer.h"

// ***FUNCTIONS*********************************************************************** 

#ifdef NO_INLINE
#  include "TContainer.i.C"
#endif

TBaseObject *
DisposeContainerEntry(TBaseObject *Entry, bool AutoDisposeEntry, bool AutoDisposeObject)

{
    TBaseObject *InstancePtr = 0;

    if (Entry != 0 AND AutoDisposeEntry)
    {
        if (Entry != (InstancePtr = Entry->GetInstancePtr()))
            delete Entry;
        else
            InstancePtr = Entry;
        if (AutoDisposeObject)
        {
            delete InstancePtr;
            Entry = 0;
        }
    }    
    return Entry;
}

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TContainer::TContainer(long theFlags) :

ParentClass(theFlags)

{
}

// =================================================================================== 

TContainer::operator TContainer * (void)

{
    return this;
}

// =================================================================================== 

bool
TContainer::_AddCopies(TContainer *aContainer, int CompareCode, bool Unique)

{
    TEnumeratorPrimitive CItem(aContainer);

    CItem.Enumerate();
    while (CItem())
    {
        TBaseObject *NewObject;
        bool         Add = FALSE;

        if (Unique)
            Add = !_Contains(CItem, CompareCode);
        else
            Add = TRUE;
        if (Add)
        {
            if ((NewObject = CItem->Copy()) == 0)
            {
                Error("AddCopies: cannot duplicate %s", (char *) CItem->GetInstanceName());
                return FALSE;
            }
            _Insert(NewObject, TRUE);
        }
        CItem++;
    }
    return TRUE;
}

// =================================================================================== 

bool
TContainer::CheckSort(int CompareCode)

{
    TEnumeratorPrimitive  CItem(this);
    TBaseObject          *KeyItem = 0;
    
    CItem.Enumerate();
    while (CItem())
    {
        /*
        if (KeyItem)
            Print("CheckSort: compare '%s' to '%s'", 
                  (char *) CItem.GetCurrentContainerEntry()->GetInstanceName(),
                  (char *) KeyItem->GetInstanceName());
                  */
        if (KeyItem AND CompareItems(CItem.GetCurrentContainerEntry(), KeyItem, CompareCode, TRUE) < 0)
        {
            Warning("CheckSort: entry '%s' is before '%s'", 
                    (char *) KeyItem->GetInstanceName(),
                    (char *) CItem.GetCurrentContainerEntry()->GetInstanceName());
        }
        KeyItem = CItem.GetCurrentContainerEntry();
        CItem++;
    }
    return TRUE;
}

// ===================================================================================

int 
TContainer::CompareItems(TBaseObject *Key, TBaseObject *Local, 
                         int CompareCode, bool CheckReference)

{
    int Result;

    if ((Result = Key->Compare(Local, CompareCode)) == 0)
        if (CheckReference)
            Result = (int) ((char *) Key->GetInstancePtr() - 
                            (char *) Local->GetInstancePtr());
        else
            Result = 0;

    /*
    Print("compare key '%s' to entry '%s' result:%d", (char *) Key->GetInstanceName(),
          (char *) Local->GetInstanceName(), Result);
          */
    return Result;
}

// =================================================================================== 

bool
TContainer::_Contains(TBaseObject *Object, int CompareCode)

{    
    return _Find(Object, CompareCode) != 0;
}

// =================================================================================== 

TBaseObject *
TContainer::_DeleteByReference(TBaseObject *, int)

{
    NotOverloaded("DeleteByReference");
    return 0;
}

// =================================================================================== 

void
TContainer::DumpEntries(void)

{
    if (GetNrOfElements() > 0)
    {
        if (WarnAboutEntriesOnDispose())
            Warning("%ld entries have not been deleted", GetNrOfElements());
        if (DumpEntriesOnDispose())
        {
            TEnumeratorPrimitive CItem(this);
        
            CItem.Enumerate();
            while (CItem())
            {
                Print("entry: %s", (char *) CItem.GetCurrentContainerEntry()->GetInstanceName());
                CItem++;
            }
        }
    }
}

// =================================================================================== 

void
TContainer::Enumerate(TEnumeratorPrimitive *)

{
    NotOverloaded("Enumerate");
}

// =================================================================================== 

TBaseObject *
TContainer::_Find(TBaseObject *, int)

{
    NotOverloaded("Find");
    return 0;
}

// =================================================================================== 

TBaseObject *
TContainer::GetCurrent(TEnumeratorPrimitive *)

{
    NotOverloaded("GetCurrent");
    return 0;
}

// =================================================================================== 

long
TContainer::GetNrOfElements(void) const

{
    NotOverloaded("GetNrOfElements");
    return -1;
}

// =================================================================================== 

bool
TContainer::_Insert(TBaseObject *, bool, int)

{
    NotOverloaded("Insert");
    return FALSE;
}

// =================================================================================== 

void
TContainer::MoveBackward(TEnumeratorPrimitive *)

{
    NotOverloaded("MoveBackward");
}

// =================================================================================== 

void
TContainer::MoveForward(TEnumeratorPrimitive *)

{
    NotOverloaded("MoveForward");
}

// =================================================================================== 

void
TContainer::Select(TSelectorPrimitive *)

{
    NotOverloaded("Select");
}

// =================================================================================== 

bool
TContainer::WhileEnumeration(TEnumeratorPrimitive *)

{
    NotOverloaded("WhileEnumeration");
    return FALSE;
}

// =================================================================================== 

bool
TContainer::WhileSelection(TSelectorPrimitive *)

{
    NotOverloaded("WhileSelection");
    return FALSE;
}

// =================================================================================== 
#undef ParentClass
