/*      file     : common/TOrthogonalBisection.h
        project  : General Toolbox
        author   : Marcus Rickert
        objects  : TOrthogonalBisection
        RCS      : $Id: TOrthogonalBisection.h,v 1.14 2005/03/25 11:04:59 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TORTHOGONALBISECTION_H
#define TORTHOGONALBISECTION_H

// ***DEFINES*************************************************************************

#define BF_BEST_OF_TWO          1
#define BF_SWITCH_CLUSTERS      2

#define DEFAULT_BISECTION_FLAGS 0

// ***INCLUDES************************************************************************ 

#include <stdarg.h>
#include <stdio.h>

#ifdef TOOLBOX
#  include <TBaseObject.h>
#endif

// ***CONSTANTS***********************************************************************

#ifndef NIL
#define NIL (-1)
#endif

#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif

#ifndef TOOLBOX
#define AND &&
#define OR ||
#endif

#define TOB_NODE_FLAG_SCANNED    1
#define TOB_NODE_FLAG_CONNECTED  2

#define TOB_EDGE_FLAG_NO_SPLIT 1

// ***TYPES*************************************************************************** 

#ifndef TOOLBOX
typedef short unsigned int uword;
typedef int bool;
#endif

extern "C" {

    struct TOBNode
    {
        long  ID;
        double Weight;
        double X, Y;
        double NodeX, NodeY;
        uword Flags;
        int   ClusterID;
        uword CPNID;
        int   NextClusterOnCPN;
        long  Cluster;
        long  NextNodeInCluster;
        int   EdgeListBase;
    };

    struct TOBEdge
    {
        long  ID;
        long  Nodes[2];
        double Weight;
        uword Multiplicity;
        uword Flags;    
    };

    struct TOBCPN
    {
        double Performance;
        double Weight;
        long  FirstCluster;
        int   NrOfNodes;
        int   NrOfClusters;
        int   NrOfSplitEdges;
        int   NrOfEdges;
    };

    struct TOBIncidence
    {
        long  EdgeID;
        int   Next;
    };
}

// ***CLASSES************************************************************************* 

#ifdef TOOLBOX
class TOrthogonalBisection : public TBaseObject
#else
class TOrthogonalBisection
#endif

    {
    public:
        
        TOrthogonalBisection(void);
        ~TOrthogonalBisection(void);

        bool
            Allocate(int theMaxNrOfNodes, int theMaxNrOfEdges, int theMaxNrOfCPN);
        bool
            Bisection(int theCurrentNrOfCPN, 
                      int theBisectionFlags = DEFAULT_BISECTION_FLAGS);
#ifdef TOOLBOX
        virtual TString
            GetClassName(void) const;
#else
        virtual void
            Error(const char *Format ...) const;
#endif
        int
            GetCPNIndexByNodeID(long NodeID) const;
        bool
            PutCPN(double thePerformance, int Count = 1);
        bool
            PutEdge(long theID, long Node1, long Node2, double theWeight,
                    bool NoSplit = FALSE);
        bool
            PutNode(long theID, double theX, double theY, double theWeight = 0.0);
        bool
            Prepare(void);
#ifndef TOOLBOX
        void
            Print(const char *Format ...);
        void
            Print(unsigned long, const char *Format ...);
        virtual void
            Warning(const char *Format ...) const;
#endif
        bool
            WriteClustersAndNonClusters(const char *BaseFilename);
        bool
            WriteCPNDistributionFiles(const char *BaseFilename);
        void
            WriteStatHeader(FILE *Ascii) const;
        void
            WriteStatLine(FILE *Ascii) const;

    private:

        int MaxNrOfNodes;
        int MaxNrOfEdges;
        int MaxNrOfCPN;

        int NrOfNodes;
        int NrOfEdges;
        int NrOfCPN;
        int NrOfClusters;
        int NrOfIncidences;

        int CurrentNrOfCPN;
        int BisectionFlags;

        TOBNode       *Nodes;
        TOBEdge       *Edges;
        TOBCPN        *CPN;
        TOBIncidence  *Incidences;
        
        // statistics

        int NrOfSplitEdges;
        int MinNrOfNodesPerCPN;
        int MaxNrOfNodesPerCPN;
        int NrOfShiftImprovements;

        double MinPerformance;
        double MaxPerformance;
        double MinWeight;
        double MaxWeight;
        double AverageWeightPerPerformance;
        double MinWeightPerPerformance;
        double MaxWeightPerPerformance;
        double MaxEfficiency;
        double MaxSpeedup;

        bool
            Bisect(int FirstNode, int LastNode, int FirstCPN, int LastCPN);
        bool
            CheckMap(void);
        bool
            Cluster(void);
        bool
            CompressMap(void);
        bool
            ComputeStatistics(void);
        long
            GetNodeIndexByID(long theID) const;
        void
            InsertIncidentEdge(long PEdgeID, long PNodeID);
        inline TOBNode &
            SortedNodes(int Index) const;
    };

// ***EXTERNALS*********************************************************************** 

extern long *SortIndex;

// ***INLINE METHODS******************************************************************
#ifndef NO_INLINE
#include "TOrthogonalBisection.i.C"
#endif
// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
