/*      file     : common/TStream.C
        project  : General Toolbox
        author   : Marcus Rickert
        RCS      : $Id: TStream.C,v 1.8 2005/03/25 11:05:00 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************

#include <string.h>
#include <errno.h>

#include <TDebug.h>
#include <TString.h>
#include <TStream.h>

// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#include <TStream.i.C>
#endif

// =================================================================================== 

#include "TStream.Bodies.C"

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TStream::TStream(char *theDelimiters, long theMaxInputLineLength)

{
    File = 0;
    IsPipe = FALSE;
    LineNr = INVALID;
    LinePos = INVALID;

    Delimiters = theDelimiters;
    CurrentDelimiters = 0;
    CurrentToken = 0;
    MaxInputLineLength = theMaxInputLineLength;
    
    InputLine = new char[MaxInputLineLength + 1];
}

// =================================================================================== 

TStream::~TStream(void)

{
    Close();
    DeleteObjectArray(InputLine);
}

// =================================================================================== 

void
TStream::ClearError(void)

{
    if (File)
        clearerr(File);
}

// =================================================================================== 

void
TStream::Close(void)

{
    if (File)
    {
//      Print("closing file...");
        if (IsPipe)      
            pclose(File);
        else
            fclose(File);
        File = 0;
    }
}

// ===================================================================================

/**
   This method should be generated from a template!
 */

bool
TStream::Decode(LongID &Value, long Count, long)

{
    if (File == 0)
    {
        Error("Decode: no file opened");
        return FALSE;
    }
    if (Binary)
    {
        long BytesToBeRead;
        long BytesRead;

        BytesToBeRead = (LongID) (Count * sizeof(LongID));
        //      Print("read %ld bytes", BytesToBeRead);
        BytesRead = (LongID) fread((void *)  &Value, BytesToBeRead, 1, File);
        if (BytesRead < 1)
        {
            Warning("Decode: too few bytes read");
            return FALSE;
        }
    }
    else
    {
        long i;

        i = 0;
        while (i < Count)
        {
            if (CurrentToken == 0)
            {
                Error("Decode: no current token");
                return FALSE;
            }
            if (sscanf(CurrentToken, LONG_ID_PRINTF_FORMAT, ((LongID *) &Value) + i) != 1)
            {
                Error("illegal value '%s', expected unsigned long", CurrentToken);
                return FALSE;
            }   
            ReadToken();
            i++;
        }
    }
    return TRUE;
}

// ===================================================================================


bool
TStream::Encode(const LongID &Value, long Count, long) const

{
    if (File == 0)
    {
        Error("Decode: no file opened");
        return FALSE;
    }

    if (Binary)
    {
        long BytesToBeWritten;
        long BytesWritten;

        BytesToBeWritten = (long) (Count * sizeof(LongID));
        BytesWritten = (long) fwrite((void *)  &Value, BytesToBeWritten, 1, File);
        if (BytesWritten < 1)
        {
            Error("Encode: too few bytes written");
            return FALSE;
        }
    }
    else
    {
        long i = 0;

        while (i < Count)
        {
            fprintf(File, LONG_ID_PRINTF_FORMAT, * (((LongID *) &Value) + i));
            fprintf(File, " ");
            i++;
        }
    }
    return TRUE;
}

// =================================================================================== 

bool
TStream::EndOfFile(void) const

{
    return ((File == 0) OR feof(File)) AND (IsBinary() OR InputLine[LinePos] == 0);
}

// =================================================================================== 

void
TStream::Flush(void)

{
    if (File)
        fflush(File);
}

// =================================================================================== 

TString
TStream::GetClassName(void) const

{
    return "TStream";
}

// =================================================================================== 

TString
TStream::GetInstanceName(void) const

{
    char Line[256];

    if (File)
    {
        if (IsPipe)
            sprintf(Line, "PIPE %s:%ld", (const char *) Filename, LineNr);
        else
            sprintf(Line, "%s:%ld", (const char *) Filename, LineNr);
        return Line;
    }
    else
        return "no file";
}

// =================================================================================== 

long
TStream::GetPercentage(void)

{
    long Position, Size;

    if (File == 0 OR IsPipe)
        return -1;
    Position = ftell(File);
    fseek(File, 0, SEEK_END);
    Size = ftell(File);
    fseek(File, Position, SEEK_SET);
    return (long) (100.0 * (double) Position / (double) Size);
}

// =================================================================================== 

bool
TStream::IsBinary(void) const

{
    return Binary;
}

// =================================================================================== 

bool
TStream::Open(const char *theFilename, const char *Mode, bool BinaryFlag)

{
    int Length;

    Filename = theFilename;
    Binary = BinaryFlag;
    Length = Filename.Length();
    IsPipe = (Length > 3 AND
              theFilename[Length - 3] == '.' AND
              theFilename[Length - 2] == 'g' AND
              theFilename[Length - 1] == 'z');
    if (IsPipe)
    {
        char Command[512];

        if (strchr(Mode, 'r') != 0)
            sprintf(Command, "gunzip -c %s", Filename());
        else
            sprintf(Command, "gzip > %s", Filename());
        if ((File = popen(Command, Mode)) == 0)
        {
            Error("IO-Error %d while opening pipe '%s'", errno, Command);
            return FALSE;
        }
        Print("pipe opened");
    }
    else
    {
        if ((File = fopen(Filename, Mode)) == 0)
        {
            Error("IO-Error %d while opening file %s", errno, Filename());
            return FALSE;
        }
    }
    LineNr = 0;
    if (strchr(Mode, 'r') AND !Binary)
        ReadLine();
    //ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(short int &Value)

{
    if (CurrentToken == 0)
        return FALSE;
    if (sscanf(CurrentToken, "%hd", &Value) != 1)
    {
        Error("illegal value '%s', expected short int", CurrentToken);
        return FALSE;
    }
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(short unsigned int &Value)

{
    if (CurrentToken == 0)
        return FALSE;
    if (sscanf(CurrentToken, "%hd", &Value) != 1)
    {
        Error("illegal value '%s', expected unsigned short int", CurrentToken);
        return FALSE;
    }
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(long &Value)

{
    if (CurrentToken == 0)
        return FALSE;
    if (sscanf(CurrentToken, "%ld", &Value) != 1)
    {
        Error("illegal value '%s', expected integer", CurrentToken);
        return FALSE;
    }
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(LongID &Value)

{
    if (CurrentToken == 0)
        return FALSE;
    if (sscanf(CurrentToken, LONG_ID_PRINTF_FORMAT, &Value) != 1)
    {
        Error("illegal value '%s', expected LongID", CurrentToken);
        return FALSE;
    }
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(double &Value)

{
    if (CurrentToken == 0)
        return FALSE;
    if (sscanf(CurrentToken, "%lf", &Value) != 1)
    {
        Error("illegal value '%s', expected double", CurrentToken);
        return FALSE;
    }
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(float &Value)

{
    if (CurrentToken == 0)
        return FALSE;
    if (sscanf(CurrentToken, "%f", &Value) != 1)
    {
        Error("illegal value '%s', expected float", CurrentToken);
        return FALSE;
    }
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::Read(char *Buffer)

{
    if (CurrentToken == 0)
        return FALSE;
    strcpy(Buffer, CurrentToken);
    ReadToken();
    return TRUE;
}

// =================================================================================== 

bool
TStream::ReadLine(void)

{
    InputLine[0] = 0;
    while (!(EndOfFile() OR InputLine[0]))
//      if (fscanf(File,"%[^\n]", InputLine) > 0)
        if (fgets(InputLine, (int) MaxInputLineLength, File) != 0)
        {
            LineNr++;
            LinePos = 0;
            CurrentDelimiters = Delimiters;
            SkipWhiteChars();
            if (InputLine[LinePos])
            {
                if (InputLine[LinePos] == COMMENT_CHAR)
                {
                    debug(Print(DM_PARSE,"comment line read: '%s'", InputLine));
                    InputLine[0] = 0;
                }
            }
            else
            {
                debug(Print(DM_PARSE, "empty input line!"));
                InputLine[0] = 0;
            }
        }
#ifdef DPARSER
    if (InputLine[LinePos])
        Print("current line: '%s'", InputLine+LinePos);
#endif
    return TRUE;
}

// =================================================================================== 

const char *
TStream::ReadToken(void)

{
    CurrentToken = 0;
    CurrentDelimiters = Delimiters;
    while ((CurrentToken == 0 OR *CurrentToken == 0) AND !EndOfFile())
    {
        if (InputLine[LinePos] == 0)
            if (!ReadLine())
            {
                Warning("ReadToken: error in ReadLine");
                return 0;
            }
        if (InputLine[LinePos] == QUOTATION_CHAR)
        {
            CurrentDelimiters = QUOTATION_CHAR_DELIMITER;
            LinePos++;
        }
        CurrentToken = InputLine + LinePos;
        while (InputLine[LinePos] AND InputLine[LinePos] != NEWLINE_CHAR AND

               strchr(CurrentDelimiters, InputLine[LinePos]) == 0

               AND InputLine[LinePos] != TAB_CHAR)
            LinePos++;
        if (InputLine[LinePos])
        {
            InputLine[LinePos] = 0;
            LinePos++;
            SkipWhiteChars();
        }
        IfDebug(DM_PARSE, Print("token read: '%s'", CurrentToken));
    }
#ifdef DPARSER
    if (CurrentToken)
        Print("current token: '%s'", CurrentToken);
    else
        Print("no current token!");
#endif
    return CurrentToken;
}

// =================================================================================== 

bool
TStream::Write(const char *Format, ...)

{
    va_list argptr;

    if (File == 0)
    {
        Error("Write: no file opened");
        return FALSE;
    }
    va_start(argptr, Format);
    vfprintf(File, Format, argptr);
    return TRUE;
}

// =================================================================================== 
#undef ParentClass




