/*      file     : graphx/grDraw.c
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: grDraw.c,v 1.3 2005/03/25 11:06:10 mr Exp $
*/

#include <graphx.h>

void grClearScreen(grWindow *theWindow,int color) {
#ifdef DEBUG
  SETFUNC("void grClearScreen(grWindow *,int)");
  if (color < 0) ERROR("Trying to clear screen with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to clear screen with a"
    " color number higher than the actual number of colors");
#endif
  grDrawFillBox(theWindow,0,0,theWindow->MAXX+1,theWindow->MAXY+1,color);
}

void grDrawPoint(grWindow *theWindow,int x,int y,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawPoint(grWindow *,int,int,int)");
  if (color < 0) ERROR("Trying to draw point with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw point with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawPoint(theWindow->dpy,theWindow->wd,theWindow->gc,x,y);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawLine(grWindow *theWindow,int x1,int y1,int x2,int y2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawLine(grWindow *,int,int,int,int,int)");
  if (color < 0) ERROR("Trying to draw line with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw line with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawLine(theWindow->dpy,theWindow->wd,theWindow->gc,
    x1,y1,x2,y2);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawBox(grWindow *theWindow,int x1,int y1,int x2,int y2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawBox(grWindow *,int,int,int,int,int)");
  if (color < 0) ERROR("Trying to draw box with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw box with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawRectangle(theWindow->dpy,theWindow->wd,theWindow->gc,
    min(x1,x2),min(y1,y2),max(x1,x2)-min(x1,x2),max(y1,y2)-min(y1,y2));
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillBox(grWindow *theWindow,int x1,int y1,int x2,int y2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillBox(grWindow *,int,int,int,int,int)");
  if (color < 0) ERROR("Trying to fill box with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill box with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XFillRectangle(theWindow->dpy,theWindow->wd,theWindow->gc,
    min(x1,x2),min(y1,y2),max(x1,x2)-min(x1,x2),max(y1,y2)-min(y1,y2));
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawArc(grWindow *theWindow,int x,int y,int r,double a1,
  double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawArc(grWindow *,int,int,int,double,double,int)");
  if (r <= 0) ERROR("Trying to draw arc with a non-positive radius");
  if (color < 0) ERROR("Trying to draw arc with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw arc with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawArc(theWindow->dpy,theWindow->wd,theWindow->gc,
    x-r,y-r,2*r,2*r,min(ANGLE(a1),ANGLE(a2)),abs(ANGLE(a1)-ANGLE(a2)));
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillArc(grWindow *theWindow,int x,int y,int r,double a1,
  double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillArc(grWindow *,int,int,int,double,double,int)");
  if (r <= 0) ERROR("Trying to fill circle with a non-positive radius");
  if (color < 0) ERROR("Trying to fill circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill circle with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XFillArc(theWindow->dpy,theWindow->wd,theWindow->gc,
    x-r,y-r,2*r,2*r,min(ANGLE(a1),ANGLE(a2)),abs(ANGLE(a1)-ANGLE(a2)));
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawCircle(grWindow *theWindow,int x,int y,int r,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawCircle(grWindow *,int,int,int,int)");
  if (r <= 0) ERROR("Trying to draw circle with a non-positive radius");
  if (color < 0) ERROR("Trying to draw circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw circle with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawArc(theWindow->dpy,theWindow->wd,theWindow->gc,
    x-r,y-r,2*r,2*r,0,360*64);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillCircle(grWindow *theWindow,int x,int y,
  int r,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillCircle(grWindow *,int,int,int,int)");
  if (r <= 0) ERROR("Trying to fill circle with a non-positive radius");
  if (color < 0) ERROR("Trying to fill circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill circle with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XFillArc(theWindow->dpy,theWindow->wd,theWindow->gc,
    x-r,y-r,2*r,2*r,0,360*64);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawEllipticArc(grWindow *theWindow,int x,int y,
  int a,int b,double a1,double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawEllipticArc(grWindow *,int,int,int,int,double,double,int)");
  if (a <= 0) ERROR("Trying to draw elliptic arc with a non-positive x-width");
  if (b <= 0) ERROR("Trying to draw elliptic arc with a non-positive y-width");
  if (color < 0) ERROR("Trying to draw elliptic arc with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw ellipse with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawArc(theWindow->dpy,theWindow->wd,theWindow->gc,x-a,y-b,2*a,2*b,
    min(ANGLE(a1),ANGLE(a2)),abs(ANGLE(a1)-ANGLE(a2)));
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillEllipticArc(grWindow *theWindow,int x,int y,
  int a,int b,double a1,double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillEllipticArc(grWindow *,int,int,int,int,double,double,int)");
  if (a <= 0) ERROR("Trying to fill elliptic arc with a non-positive x-width");
  if (b <= 0) ERROR("Trying to fill elliptic arc with a non-positive y-width");
  if (color < 0) ERROR("Trying to fill elliptic arc with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill ellipse with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XFillArc(theWindow->dpy,theWindow->wd,theWindow->gc,x-a,y-b,2*a,2*b,
    min(ANGLE(a1),ANGLE(a2)),abs(ANGLE(a1)-ANGLE(a2)));
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawEllipse(grWindow *theWindow,int x,int y,int a,int b,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawEllipse(grWindow *,int,int,int,int,int)");
  if (a <= 0) ERROR("Trying to draw ellipse with a non-positive x-width");
  if (b <= 0) ERROR("Trying to draw ellipse with a non-positive y-width");
  if (color < 0) ERROR("Trying to draw ellipse with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw ellipse with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color%theWindow->noColors].pixel);
  XDrawArc(theWindow->dpy,theWindow->wd,theWindow->gc,
    x-a,y-b,2*a,2*b,0,360*64);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillEllipse(grWindow *theWindow,int x,int y,int a,int b,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillEllipse(grWindow *,int,int,int,int,int)");
  if (a <= 0) ERROR("Trying to fill ellipse with a non-positive x-width");
  if (b <= 0) ERROR("Trying to fill ellipse with a non-positive y-width");
  if (color < 0) ERROR("Trying to fill ellipse with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill ellipse with a"
    " color number higher than the actual number of colors");
#endif
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color].pixel);
  XFillArc(theWindow->dpy,theWindow->wd,theWindow->gc,
    x-a,y-b,2*a,2*b,0,360*64);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}

void grDrawFillPolygon(grWindow *theWindow,XPoint *Points, int NrOfPoints,
                       int shape, int mode,int color) {
  XSetForeground(theWindow->dpy,theWindow->gc,
    theWindow->colors[color].pixel);
  XFillPolygon(theWindow->dpy, theWindow->wd, theWindow->gc,
               Points, NrOfPoints, shape, mode);
  if (theWindow->speed == GrSlow) XFlush(theWindow->dpy);
}
