/*      file     : graphx/grDrawPS.C
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: grDrawPS.c,v 1.2 2005/03/25 11:06:10 mr Exp $
*/

#include <graphx.h>

void grClearScreenPS(grWindow *theWindow,int color) {
#ifdef DEBUG
  SETFUNC("void grClearScreenPS(grWindow *,int)");
  if (color < 0) ERROR("Trying to clear PostScript screen with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to PostScript clear screen with a"
    " color number higher than the actual number of colors");
#endif
  grClosePostScript(theWindow);
  grOpenPostScript(theWindow,theWindow->psFileName);
  grDrawFillBoxPS(theWindow,theWindow->Xmin,theWindow->Ymin,
    theWindow->Xmax,theWindow->Ymax,color);
}

void grDrawPointPS(grWindow *theWindow,double x,double y,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawPointPS(grWindow *,double,double,int)");
  if (color < 0) ERROR("Trying to draw point with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw point with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  grSetGreyPS(theWindow, color);
  fprintf(theWindow->psFile,"%g %g b\n",PSX(theWindow,x), PSY(theWindow,y));
}

void grDrawLinePS(grWindow *theWindow,double x1,double y1,double x2,
  double y2,int col) {
#ifdef DEBUG
  SETFUNC("void grDrawLinePS(grWindow *,double,double,double,double,int)");
  if (col < 0) ERROR("Trying to draw line with a negative color number");
  if (col > theWindow->noColors) WARNING("Trying to draw line with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)col)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,x1),
    PSY(theWindow,y1));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,x2),
    PSY(theWindow,y2));
  fprintf(theWindow->psFile,"stroke\n");
}

void grDrawBoxPS(grWindow *theWindow,double x1,double y1,
  double x2,double y2,int col) {
#ifdef DEBUG
  SETFUNC("void grDrawBoxPS(grWindow *,double,double,double,double,int)");
  if (col < 0) ERROR("Trying to draw box with a negative color number");
  if (col > theWindow->noColors) WARNING("Trying to draw box with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)col)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,min(x1,x2)),
    PSY(theWindow,min(y1,y2)));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,max(x1,x2)),
    PSY(theWindow,min(y1,y2)));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,max(x1,x2)),
    PSY(theWindow,max(y1,y2)));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,min(x1,x2)),
    PSY(theWindow,max(y1,y2)));
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"stroke\n");
}

void grDrawFillBoxPS(grWindow *theWindow,double x1,double y1,double x2,
  double y2,int col) {
#ifdef DEBUG
  SETFUNC("void grDrawFillBoxPS(grWindow *,double,double,double,double,int)");
  if (col < 0) ERROR("Trying to fill box with a negative color number");
  if (col > theWindow->noColors) WARNING("Trying to fill box with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)col)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,min(x1,x2)),
    PSY(theWindow,min(y1,y2)));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,max(x1,x2)),
    PSY(theWindow,min(y1,y2)));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,max(x1,x2)),
    PSY(theWindow,max(y1,y2)));
  fprintf(theWindow->psFile,"%g %g lineto\n",PSX(theWindow,min(x1,x2)),
    PSY(theWindow,max(y1,y2)));
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"fill\n");
}

void grDrawArcPS(grWindow *theWindow,double x,double y,double r,
  double a1,double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawArcPS(grWindow *,double,double,double,double,double,int)");
  if (r <= 0) ERROR("Trying to draw circle with a non-positive radius");
  if (color < 0) ERROR("Trying to draw circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw circle with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,x),
    PSY(theWindow,y));
  fprintf(theWindow->psFile,"%g %g %g %g %g arc\n",PSX(theWindow,x),
    PSY(theWindow,y),PSLX(theWindow,r),a1,a2);
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"stroke\n");
}

void grDrawFillArcPS(grWindow *theWindow,double x,double y,double r,
  double a1,double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillArcPS(grWindow *,double,double,double,double,double,int)");
  if (r <= 0) ERROR("Trying to fill circle with a non-positive radius");
  if (color < 0) ERROR("Trying to fill circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill circle with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",
    ((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,x),
    PSY(theWindow,y));
  fprintf(theWindow->psFile,"%g %g %g %g %g arc\n",PSX(theWindow,x),
    PSY(theWindow,y),PSLX(theWindow,r),a1,a2);
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"fill\n");
}

void grDrawCirclePS(grWindow *theWindow,double x,double y,double r,
  int color) {
#ifdef DEBUG
  SETFUNC("void grDrawCirclePS(grWindow *,double,double,double,int)");
  if (r <= 0) ERROR("Trying to draw circle with a non-positive radius");
  if (color < 0) ERROR("Trying to draw circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw circle with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"%g %g %g 0 360 arc\n",PSX(theWindow,x),
    PSY(theWindow,y),PSLX(theWindow,r));
  fprintf(theWindow->psFile,"stroke\n");
}

void grDrawFillCirclePS(grWindow *theWindow,double x,double y,double r,
  int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillCirclePS(grWindow *,double,double,double,int)");
  if (r <= 0) ERROR("Trying to fill circle with a non-positive radius");
  if (color < 0) ERROR("Trying to fill circle with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill circle with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",
    ((double)color)/theWindow->noColors);
  fprintf(theWindow->psFile,"%g %g %g 0 360 arc\n",PSX(theWindow,x),
    PSY(theWindow,y),PSLX(theWindow,r));
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"fill\n");
}

void grDrawEllipticArcPS(grWindow *theWindow,double x,double y,double a,
  double b,double a1,double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawEllipticArc(grWindow *,double,double,double,double,double,double,int)");
  if (a <= 0) ERROR("Trying to draw elliptic arc with a non-positive x-width");
  if (b <= 0) ERROR("Trying to draw elliptic arc with a non-positive y-width");
  if (color < 0) ERROR("Trying to draw elliptic arc with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw ellipse with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"1 %g scale\n",b/a);
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,x),
    PSY(theWindow,y));
  fprintf(theWindow->psFile,"%g %g %g %g %g arc\n",PSX(theWindow,x),
    PSY(theWindow,a/b*y),PSLX(theWindow,a),a1,a2);
  fprintf(theWindow->psFile,"stroke\n");
  fprintf(theWindow->psFile,"1 %g scale\n",a/b);
}

void grDrawFillEllipticArcPS(grWindow *theWindow,double x,double y,double a,
  double b,double a1,double a2,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillEllipticArcPS(grWindow *,double,double,double,double,double,double,int)");
  if (a <= 0) ERROR("Trying to fill elliptic arc with a non-positive x-width");
  if (b <= 0) ERROR("Trying to fill elliptic arc with a non-positive y-width");
  if (color < 0) ERROR("Trying to fill elliptic arc with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill ellipse with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"1 %g scale\n",b/a);
  fprintf(theWindow->psFile,"%g %g moveto\n",PSX(theWindow,x),
    PSY(theWindow,y));
  fprintf(theWindow->psFile,"%g %g %g %g %g arc\n",PSX(theWindow,x),
    PSY(theWindow,a/b*y),PSLX(theWindow,a),a1,a2);
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"fill\n");
  fprintf(theWindow->psFile,"1 %g scale\n",a/b);
}

void grDrawEllipsePS(grWindow *theWindow,double x,double y,double a,
  double b,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawEllipsePS(grWindow *,double,double,double,double,int)");
  if (a <= 0) ERROR("Trying to draw ellipse with a non-positive x-width");
  if (b <= 0) ERROR("Trying to draw ellipse with a non-positive y-width");
  if (color < 0) ERROR("Trying to draw ellipse with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to draw ellipse with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"1 %g scale\n",b/a);
  fprintf(theWindow->psFile,"%g %g %g 0 360 arc\n",PSX(theWindow,x),
    PSY(theWindow,a/b*y),PSLX(theWindow,a));
  fprintf(theWindow->psFile,"stroke\n");
  fprintf(theWindow->psFile,"1 %g scale\n",a/b);
}

void grDrawFillEllipsePS(grWindow *theWindow,double x,double y,double a,
  double b,int color) {
#ifdef DEBUG
  SETFUNC("void grDrawFillEllipsePS(grWindow *,double,double,double,double,int)");
  if (a <= 0) ERROR("Trying to fill ellipse with a non-positive x-width");
  if (b <= 0) ERROR("Trying to fill ellipse with a non-positive y-width");
  if (color < 0) ERROR("Trying to fill ellipse with a negative color number");
  if (color > theWindow->noColors) WARNING("Trying to fill ellipse with a"
    " color number higher than the actual number of colors");
#endif
  if (theWindow->psFile == NULL) return;
  fprintf(theWindow->psFile,"newpath\n");
  fprintf(theWindow->psFile,"%g setgray\n",((double)color)/(theWindow->noColors-1));
  fprintf(theWindow->psFile,"1 %g scale\n",b/a);
  fprintf(theWindow->psFile,"%g %g %g 0 360 arc\n",PSX(theWindow,x),
    PSY(theWindow,a/b*y),PSLX(theWindow,a));
  fprintf(theWindow->psFile,"closepath\n");
  fprintf(theWindow->psFile,"fill\n");
  fprintf(theWindow->psFile,"1 %g scale\n",a/b);
}

void grSetGreyPS(grWindow *theWindow, int Color) /* MR */

{
    if (Color != theWindow->psColor)
    {
        theWindow->psColor = Color;
        fprintf(theWindow->psFile, "%g setgray\n", ((double) Color) / (theWindow->noColors - 1));
    }
}
