/*      file     : graphx/mygrtest.c
        project  : graphical toolbox
        author   : Michael Olesen
        comments : simple classes for using X11
        RCS      : $Id: mygrtest.c,v 1.2 2005/03/25 11:06:10 mr Exp $
*/

#include <graphx.h>

char fontFamily[5][30] = {
  "Courier",
  "Helvetica",
  "New Century Schoolbook",
  "Times",
  "Symbol"
};

char fontWeight[2][30] = {
  "Medium",
  "Bold"
};

char fontShape[2][30] = {
  "Regular",
  "Italic"
};

void main(int argc,char **argv) {
  char fontName[80];
  int m,f,w,s,size;
  double x,y;
  grWindow *theWindow;

  if (argc != 5) {
    fprintf(stderr,"Usage: showfont fontfamily fontweight fontshape fontsize.\n\a");
    exit(1);
  }

  sscanf(argv[1],"%d",&f);
  sscanf(argv[2],"%d",&w);
  sscanf(argv[3],"%d",&s);
  sscanf(argv[4],"%d",&size);
  
  theWindow = grCreateWindow("Hello Michael",500,500);
  grSetXaxis(theWindow,0,10);
  grSetYaxis(theWindow,0,10);

  if (theWindow == NULL) {
    fprintf(stderr,"Unable to create graphics window.\n");
    exit(1);
  }
  
  sprintf(fontName,"%s-%s-%s-%dpt",fontFamily[f],fontWeight[w],
    fontShape[s],size);
  
  if (!grSetFont(theWindow,f,w,s,size)) {
    fprintf(stderr,"Unable to load font %s.\n\a",fontName);
    exit(1);
  );
  
  grX11SetSpeed(theWindow,GrFast);
  
  grX11WaitUntilExposed(theWindow);
  
  grClearScreen(theWindow,255);
  
  m = 0;
  while(1) {
    x = rand()%1000/100.0;
    y = rand()%1000/100.0;
    grUsrAdjustTextAlignment(theWindow,x,y,"Hello Michael Olesen");
    grUsrDrawString(theWindow,x,y,"Hello Michael Olesen",0);
    grDrawLine(theWindow,X(theWindow,x)-10,Y(theWindow,y),
      X(theWindow,x)+10,Y(theWindow,y),0);
    grDrawLine(theWindow,X(theWindow,x),Y(theWindow,y)-10,
      X(theWindow,x),Y(theWindow,y)+10,0);
    grX11Flush(theWindow);
    printf("TextBoundaries: %d < x < %d and %d < y < %d.\n",
      grLeftTextBoundary(theWindow,X(theWindow,x),"Hello Michael Olesen"),
      grRightTextBoundary(theWindow,X(theWindow,x),"Hello Michael Olesen"),
      grTopTextBoundary(theWindow,Y(theWindow,y),"Hello Michael Olesen"),
      grBottomTextBoundary(theWindow,Y(theWindow,y),"Hello Michael Olesen"));
    printf("TextBoundaries: %g < x < %g and %g < y < %g.\n",
      grUsrLeftTextBoundary(theWindow,x,"Hello Michael Olesen"),
      grUsrRightTextBoundary(theWindow,x,"Hello Michael Olesen"),
      grUsrTopTextBoundary(theWindow,y,"Hello Michael Olesen"),
      grUsrBottomTextBoundary(theWindow,y,"Hello Michael Olesen"));
    printf("-----------------------------------------\n");
    sleep(1);
    m++;
    m %= 500;
  }
}
