/*      file     : parallel/TEventHandler.t.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TEventHandler.t.C,v 1.6 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#include <gcc_pragma_interface.h>

// ***METHODS************************************************************************* 

template <class T>
TEventHandler<T>::TEventHandler(T *theLocalHandler, 
                                TEventHandlerPrimitive *theRelayHandler) :

TEventHandlerPrimitive(theRelayHandler)

{
    EventMethods = 0;
    LocalHandler = theLocalHandler;
}

// =================================================================================== 

template <class T>
TEventHandler<T>::~TEventHandler(void)

{
    DeleteObjectArray(EventMethods);
}

// =================================================================================== 

template <class T>
TString
TEventHandler< T >::GetClassName(void) const

{
    char line[256];

    sprintf(line, "TEventHandler<%s>", (char *) LocalHandler->GetClassName());
    return line;
}

// =================================================================================== 

template <class T>
bool
TEventHandler<T>::HandleEvent(TEventID EventID, TBaseMessage *Message, 
                              int &Handled) const

{

    if (EventID < 0 OR EventID >= MAX_EVENTS)
        return FALSE;
    if (HandlerObjects AND HandlerObjects[EventID])
        return HandlerObjects[EventID]->HandleEvent(EventID, Message, Handled);
    else
        if (EventMethods AND EventMethods[EventID])
            return (LocalHandler->*EventMethods[EventID])(Message, Handled);
        else
        {
            Warning("event ID %d not handled", EventID);
            Handled = FALSE;
        }
    return TRUE;
}

// =================================================================================== 

template <class T>
bool
TEventHandler<T>::Register(TEventID EventID,  
                           bool (T::*EventMethod) (TBaseMessage *, int &))

{
    if (EventID >= MAX_EVENTS)
    {
        Error("cannot register event ID %d > MAX_EVENTS", EventID);
        return FALSE;
    }
    if (RelayHandler)
        if (!RelayHandler->RegisterEventHandler(EventID, this))
            return FALSE;
    if (EventMethods == 0)
    {
        int i;
        
        typedef bool (T::*PEventMethod) (TBaseMessage *, int &);

        EventMethods = new PEventMethod[MAX_EVENTS];
        for (i = 0; i < MAX_EVENTS; i++)
            EventMethods[i] = 0;
    }
    if (EventMethods[EventID])
    {
        Error("RegisterEventMethod: event ID %ld already used", EventID);
        return FALSE;
    }
    EventMethods[EventID] = EventMethod;
    return TRUE;
}

// =================================================================================== 
