/*      file     : parallel/TMessageRecipient.h
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TMessageRecipient.h,v 1.7 2005/03/25 11:06:20 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

#ifndef TMESSAGERECIPIENT_H
#define TMESSAGERECIPIENT_H

// ***INCLUDES************************************************************************ 

#include "CommonTypes.h"
#include "ParallelTypes.h"

// ***FORWARDS************************************************************************

class TBaseMessage;
class TEventHandlerPrimitive;

// ***DEFINES************************************************************************* 

#define EXIT_LOOP 2

// ***CLASSES************************************************************************* 

/** Base class for classes which are to receive messages.
  */

class TMessageRecipient

{
public:

    TMessageRecipient(void);

    virtual bool
        HandleMessage(TBaseMessage *&aMessage, bool &ExitLoop);
    void
        SetEventHandler(const TEventHandlerPrimitive *theEventHandler);

protected:
    
    /** Stub to handle an event encoded in the message.
      Each event found in a message received by TCommunication is passed to this method, 
      if the message tags match those used when registering this recipient.
      @return TRUE if successful
      @param theEventID event ID of the current event which is to decoded from 
      the message
      @param aMessage reference to an incoming message.
      @param Handled must be set to {\bf FALSE}, if an error has occurred while 
      processing the message, {\bf TRUE}, if the processing was successful, or 
      {\bf EXIT\_LOOP} if the processing was successful and the calling message loop
      is to be exited. */
    virtual bool
        HandleEvent(TEventID theEventID, TBaseMessage *theMessage, int &Handled);
    virtual bool
        HandleEventMessage(TBaseMessage *&aMessage, bool &ExitLoop);

private:
    
    const TEventHandlerPrimitive *EventHandler;
};

// ===================================================================================
// Local Variables:
// mode:c++
// End:
#endif
