/*      file     : TPLBalancer.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TPLBalancer.C,v 1.4 2005/03/25 11:06:49 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TEnvironment.h>
#include <TDebug.h>

#include <TDestination.h>
#include <TTimeStep.h>
#include <TClassInfo.h>

#include <plb.h>
#include <ParBalanceTypes.h>

#include "TPLBalancer.h"

// ***DEFINES************************************************************************* 

#define PLB_SCALE 1.0

// ***METHODS************************************************************************* 

// =================================================================================== 
#define ParentClass TBaseBalancer
// =================================================================================== 

TPLBalancer::TPLBalancer(void)

{    
    Topology = 0;
    Mail = 0;
    Precomp = 0;
    PlbDim = 0;
    ResultsMissing = FALSE;
}

// =================================================================================== 

TPLBalancer::~TPLBalancer(void)

{
    DeleteObject(Precomp);
    DeleteObject(Mail);
    DeleteObject(Topology);
}

// =================================================================================== 

bool
TPLBalancer::CheckResults(TTimeStep &TimeStep)

{
    plb_load_vec_t ExchangeVector[10];  // maximal 9 sons
    int            NrOfExchanges;
    int            CExchange;
    TCPNID         DestCPNID;

    if (ResultsMissing)
        if (Block OR (!Block AND !Precomp->isRunning()))
        {
            
            if (TimeStep > StartTimeStep)
                Print(DM_BALANCE, "balancer: started %s finished:%s", 
                      (char *) StartTimeStep.GetInstanceName(), 
                      (char *) TimeStep.GetInstanceName());
            NrOfExchanges = Precomp->ExchangeVec(ExchangeVector, PLB_GG_SG);
            for (CExchange = 0; CExchange < NrOfExchanges; CExchange++)
            {
                DestCPNID = Precomp->GetComID(ExchangeVector[CExchange].proc);
                
                if (!SetTransfer(DestCPNID, StartTimeStep, 
                                 ExchangeVector[CExchange].load / PLB_SCALE))
                {
                    Error("CheckResults: cannot set transfer load %f to %lX",
                          ExchangeVector[CExchange].load / PLB_SCALE, 
                          ExchangeVector[CExchange].proc);
                    return FALSE;
                }
            }
            ResultsMissing = FALSE;
        }
    return TRUE;
}

// =================================================================================== 

bool
TPLBalancer::ExecuteSubTimeStep(TTimeStep &TimeStep)

{
    if (!ParentClass::ExecuteSubTimeStep(TimeStep))
        return FALSE;
    if (TimeStep.GetSubStep() == 0)
    {
        if (TimeStep % Interval == 0 AND
            LocalLoad != INVALID AND LocalPerf != INVALID)
            if (!StartPrecomputation(TimeStep))
            {
                Error("ExecuteSubTimeStep: while calling precomputation");
                return FALSE;
            }
        if (!CheckResults(TimeStep))
        {
            Error("ExecuteSubTimeStep: cannot retrieve results");
            return FALSE;
        }
    }
    return TRUE;
}

// =================================================================================== 

TString
TPLBalancer::GetClassName(void) const

{
    return "TPLBalancer";
}

// =================================================================================== 

bool
TPLBalancer::StartPrecomputation(TTimeStep &TimeStep)

{
    if (ResultsMissing)
    {
        Warning("StartPrecomputation in %s: results still missing from %s -> no action",
                (char *) TimeStep.GetInstanceName(), (char *) StartTimeStep.GetInstanceName());
        return TRUE;
    }
    Mail->Init(PLB_SCALE * LocalLoad, LocalPerf, LocalTime, TimeStep);
    if (Block)  
        Precomp->Run(Mail, PlbDim);
    else
        Precomp->Start(Mail, PlbDim);
    ResultsMissing = TRUE;
    StartTimeStep = TimeStep;
    return TRUE;
}

// =================================================================================== 

bool
TPLBalancer::Run(void)

{
    TTopology   *PhysicalTopology;

    if (!ParentClass::Run())
        return FALSE;
    if ((PhysicalTopology = Parallel->GetPhysicalTopology()) == 0)
    {
        Error("Run: cannot retrieve physical topology");
        return FALSE;
    }
    Topology = new PlbTopo;
    Topology->SetToolbox(PhysicalTopology);
    Mail = new PlbBaseMail;
    Precomp = new PlbPrecomp(Mail, Topology, MTAG_PLB, PLB_PRECOMP_PREPARENONBLOCK);
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
