/*      file     : TrafficData/TAppEventHandler.C
        project  : PAMINA III
        author   : Marcus Rickert
        objects  : 
        comments : 
        RCS      : $Id: TAppEventHandler.C,v 1.2 2005/03/25 11:07:01 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TParser.h>
#include <TTimeStep.h>
#include <TClassInfo.h>
#include <TParallelObject.h>
#include <TObjectManager.h>

#include "TAppEventHandler.h"

// ***DEFINES************************************************************************* 

#define TOKEN_EVENT 0

// ***CONSTANTS***********************************************************************

char *AppEventHandlerTokens[] =

{
    "EVENT"
};

// ***TYPES*************************************************************************** 
// ***GLOBAL VARIABLES****************************************************************  
// ***LOCAL VARIABLES***************************************************************** 
// ***FUNCTIONS*********************************************************************** 
// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TAppEventHandler::TAppEventHandler(void)

{
    Parser = new TParser(1, AppEventHandlerTokens);
}

// =================================================================================== 

TAppEventHandler::~TAppEventHandler(void)

{
    DeleteObject(Parser);
}

// =================================================================================== 

void
TAppEventHandler::CloseInput(void)

{
    if (Parser)
    {
        DeleteObject(Parser);
        Print("closing event input file");
    }
}

// =================================================================================== 

bool
TAppEventHandler::ExecuteTimeStep(const TTimeStep &TimeStep)

{
    long EventTimeStep = 0;

    while (Parser AND !Parser->EndOfFile() AND TimeStep >= CurrentEventTimeStep)
    {
        if (!ReadEvent())
            return FALSE;
        if (!(Parser->FindToken(TOKEN_EVENT) AND
              Parser->ReadToken())) // skip token EVENT
        {
            if (Parser->EndOfFile())
            {
                CloseInput();
                return TRUE;
            }
            Error("ExecuteTimeStep: cannot read token EVENT");
            return FALSE;
        }
        if (!Parser->Decode(EventTimeStep))
        {
            Error("ExecuteTimeStep: cannot decode time-step");
            return FALSE;
        }
        if (EventTimeStep < CurrentEventTimeStep)
        {
            Error("error in event time-steps: last:%ld current:%ld",
                  CurrentEventTimeStep, (long) EventTimeStep);
            return FALSE;
        }
        CurrentEventTimeStep = EventTimeStep;
    }
    return TRUE;
}

// =================================================================================== 

TString
TAppEventHandler::GetClassName(void) const

{
    return "TAppEventHandler";
}

// =================================================================================== 

bool
TAppEventHandler::Open(const char *Filename)

{
    if (!Parser->Open(Filename))
    {
        Error("Open: cannot open file '%s'", Filename);
        return FALSE;
    }
    Print("event file '%s' opened", Filename);
    if (Parser->ReadToken())
        if (!(Parser->GetTokenID(TOKEN_EVENT) == TOKEN_EVENT AND
              Parser->ReadToken()))
        {
            Error("Open: cannot read first event");
            return FALSE;
        }
    if (!Parser->Decode(CurrentEventTimeStep))
    {
        Error("Open: cannot read time-step of first event");
        return FALSE;
    }
    return TRUE;
}

// =================================================================================== 

bool
TAppEventHandler::ReadEvent(void)

{
    char             ClassName[128];
    TParallelID      ParallelID;
    TClassInfo      *ClassInfo;
    TParallelObject *Object;

    if (!Parser->Read(ClassName))
    {
        Error("ReadEvent: cannot read class name");
        return FALSE;
    }
    if ((ClassInfo = ObjectManager->GetClassInfoByName(ClassName)) == 0)
    {
        Warning("ReadEvent: cannot find class name '%s'", ClassName);
        return TRUE;
    }
    if (!Parser->Decode(ParallelID))
    {
        Error("ReadEvent: cannot decode ParallelID");
        return FALSE;
    }
    if ((Object = ClassInfo->GetObjectPtr(ParallelID)) == 0)
    {
        //      Warning("ReadEvent: cannot find object %ld in class %s", ParallelID, ClassName);
        return TRUE;
    }
    return Object->ReadEvent(Parser);
}

// =================================================================================== 
#undef ParentClass
