/*      file     : partools/TBaseDataManager.C
        project  : Parallel Toolbox 2.x
        author   : Marcus Rickert
        RCS      : $Id: TBaseDataManager.C,v 1.3 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TNode.h>
#include <TEdge.h>
#include <TIncidenceInfo.h>
#include <TAdjacenceInfo.h>
#include <TObjectManager.h>

#include "TBaseDataManager.h"

// ***DEFINES************************************************************************* 
// ***CONSTANTS***********************************************************************
// ***TYPES*************************************************************************** 

typedef TSelector < TIncidenceTableEntry , TEdge > TIncidenceSelector;

// ***TYPES*************************************************************************** 
// ***GLOBAL VARIABLES****************************************************************  
// ***LOCAL VARIABLES***************************************************************** 
// ***FUNCTIONS*********************************************************************** 
// ***METHODS************************************************************************* 

#ifdef NO_INLINE
#endif

// =================================================================================== 

// =================================================================================== 
#define ParentClass TBaseObject
// =================================================================================== 

TBaseDataManager::TBaseDataManager(void) /* :

IncidenceTable(CM_AUTO_DISPOSE)
*/
{
}

// =================================================================================== 

bool
TBaseDataManager::Activate(void)

{
    debug(Print(DM_LIFESPAN, "ACTIVATE"));
    ObjectManager->RegisterContainerClass(0, TNode::ClassInfo, CLASS_NODE, this);
    ObjectManager->RegisterContainerClass(0, TEdge::ClassInfo, CLASS_EDGE, this);
    //    ObjectManager->RegisterClass(0, TIncidenceInfo::ClassInfo, CLASS_NODE, 
    //                           CLASS_INCIDENCE, this);
    return TRUE;
}

// =================================================================================== 
/*
bool
TBaseDataManager::DisposeIncidenceInfo(TParallelID NodeID)

{
    TNode           *Node;
    TParallelObject *Info;
    TInfoID          InfoID;

    if ((Node = (TNode *) TNode::ClassInfo->GetObjectPtr(NodeID)) == 0)
    {
        Error("cannot dispose incidence info for node %ld", NodeID);
        return FALSE;
    }
    InfoID = TIncidenceInfo::ClassInfo->GetInfoID();
    if ((Info = Node->GetInfo(InfoID)))
    {
        //      Print("dispose incidence info for %s", (char *) Node->GetObjectName());
        Node->SetInfo(InfoID, 0, TRUE);
    }
    return TRUE;
}
*/
// =================================================================================== 
/*
bool
TBaseDataManager::GenerateIncidenceInfo(TNode *Node)

{
    TIncidenceSelector  CIncidence(&IncidenceTable);
    TEdge              *Edges[MAX_CONTAINER_DEPS];
    TIncidenceInfo     *Info;
    int                 NrOfEdges;


    CIncidence.Select(TIncidenceTableEntry(Node->GetParallelID(), 0), SELECT_ID_1);
    NrOfEdges = 0;
    while (CIncidence())
    {
        Edges[NrOfEdges] = CIncidence;
        NrOfEdges++;
        CIncidence++;
    }
    if ((Info = (TIncidenceInfo *) 
         Node->GetInfo(TIncidenceInfo::ClassInfo->GetInfoID(), TRUE)) == 0)
    {
        Error("GenerateIncidenceInfo: cannot retrieve incidence info");
        return FALSE;
    }
    if (!Info->AssignEdges(NrOfEdges, Edges))
    {
        Error("cannot assign edges");
        return FALSE;
    }
    return TRUE;
}
*/
// =================================================================================== 

TString
TBaseDataManager::GetClassName(void) const

{
    return "TBaseDataManager";
}

// =================================================================================== 
/*
bool
TBaseDataManager::HandleAction(TParallelObject *Object, TObjectAction Action)

{
    if (Object->GetClassID() == TEdge::ClassInfo->GetID())
    {
        TEdge *Edge = (TEdge *) Object;

        switch (Action)
        {
        case ActionRemove:

            if (!RemoveIncidences(Edge))
            {
                Error("HandleAction: cannot remove incidences");
                return FALSE;
            }
            return TRUE;
        default:
            //      Warning("action %d ignored", (int) Action);
            ;
        } 
    }
    else if (Object->GetClassID() == TAdjacenceInfo::ClassInfo->GetID())
    {
        switch (Action)
        {
        case ActionLoaded:
        case ActionDecoded:

            if (!InsertIncidences((TEdge *) Object->GetContainer()))
            {
                Error("HandleAction: cannot insert incidences");
                return FALSE;
            }
            return TRUE;

        default:
            ;
        }
    }
    return TRUE;
}
*/
// =================================================================================== 

bool
TBaseDataManager::HandleCreateRequest(TParallelObject *&Object, const TClassInfo *ClassInfo, 
                                      TParallelID ParallelID)

{
    if (ClassInfo == TNode::ClassInfo)
        
        Object = new TNode((TClassInfo *) ClassInfo, ParallelID,
                           TIncidenceInfo::ClassInfo->GetInfoID());
    
    else if (ClassInfo == TEdge::ClassInfo)
        
        Object = new TEdge((TClassInfo *) ClassInfo, ParallelID, 
                           TAdjacenceInfo::ClassInfo->GetInfoID());
    /*
    else if (ClassInfo == TIncidenceInfo::ClassInfo)

        Object = new TIncidenceInfo((TClassInfo *) ClassInfo, ParallelID);
        */
    return TRUE;
}

// =================================================================================== 
/*
bool
TBaseDataManager::HandleLoadRequest(TParallelObject *Object, bool &Loaded)

{
    if (Object->GetClassID() == TIncidenceInfo::ClassInfo->GetID())
    {
        TNode *Node = (TNode *) Object;

        if (!GenerateIncidenceInfo(Node))
        {
            Error("HandleLoadRequest: cannot generate incidence info");
            return FALSE;
        }
        Loaded = TRUE;
    }
    return TRUE;
}
*/
// =================================================================================== 
/*
bool
TBaseDataManager::InsertIncidences(TEdge *Edge)

{
    TIncidenceTableEntry *NewEntry;
    TParallelID FromNodeID, ToNodeID;    

    if ((FromNodeID = Edge->GetFromNodeID()) == INVALID)
        Warning("InitializeEdge: cannot get FromNode ID");
    if ((ToNodeID = Edge->GetToNodeID()) == INVALID)
        Warning("InitializeEdge: cannot get ToNode ID");
    NewEntry = new TIncidenceTableEntry(FromNodeID, ToNodeID, Edge);
    if (!IncidenceTable.Insert(NewEntry, TRUE))
        delete NewEntry;
    NewEntry = new TIncidenceTableEntry(ToNodeID, FromNodeID, Edge);    
    if (!IncidenceTable.Insert(NewEntry, TRUE))
        delete NewEntry;
    return TRUE;
}
*/
// =================================================================================== 
/*
bool
TBaseDataManager::RemoveIncidences(TEdge *Edge)

{
    TParallelID           FromNodeID, ToNodeID;
    TIncidenceTableEntry *Entry;

    // Print("remove incidences for %s", (char *) Edge->GetObjectName());

    if ((FromNodeID = Edge->GetFromNodeID()) == INVALID)
        Warning("RemoveIncidences: cannot get FromNode ID");
    if ((ToNodeID = Edge->GetToNodeID()) == INVALID)
        Warning("RemoveIncidences: cannot get ToNode ID");
    if ((Entry = IncidenceTable.DeleteByReference(
        TIncidenceTableEntry(FromNodeID, ToNodeID, Edge), FALSE)))
        delete Entry;
    DisposeIncidenceInfo(FromNodeID);
    DisposeIncidenceInfo(ToNodeID);
    if ((Entry = IncidenceTable.DeleteByReference(
        TIncidenceTableEntry(ToNodeID, FromNodeID, Edge), FALSE)))
        delete Entry;
    return TRUE;
}
*/
// =================================================================================== 

bool
TBaseDataManager::Run(void)

{
    /*    
    TNode::ClassInfo->RegisterActionHandler(this);
    TEdge::ClassInfo->RegisterActionHandler(this);
    TAdjacenceInfo::ClassInfo->RegisterActionHandler(this);
    */
    return TRUE;
}

// =================================================================================== 
#undef ParentClass
