/*      file     : partools/TEdge.i.C
        project  : Parallel Toolbox 2.x
        authors  : Marcus Rickert
        RCS      : $Id: TEdge.i.C,v 1.8 2005/03/25 11:08:14 mr Exp $

        Copyright (C) 2005 Marcus Rickert

        http://www.the-rickerts.de/mr/PAMINA.html
*/

// ***INCLUDES************************************************************************ 

#include <TAdjacenceInfo.h>

// ***DEFINES************************************************************************* 

#define DefAdj(InfoID) (((InfoID) == INVALID)? DefaultAdjacenceInfoID:(InfoID))

// ***METHODS************************************************************************* 

inline TNode *
TEdge::GetFromNode(TInfoID AdjacenceInfoID) const

{
    TAdjacenceInfo *Adj;

    if ((Adj = (TAdjacenceInfo *) GetInfo(DefAdj(AdjacenceInfoID), TRUE, TRUE)) == 0)
    {
        Error("GetFromNode: cannot retrieve adjacence info");
        return 0;
    }
    return Adj->GetFromNode(FALSE);
}

// =================================================================================== 

inline TParallelID
TEdge::GetFromNodeID(TInfoID AdjacenceInfoID) const

{
    TAdjacenceInfo *Adj;

    if ((Adj = (TAdjacenceInfo *) GetInfo(DefAdj(AdjacenceInfoID), TRUE, TRUE)) == 0)
    {
        Error("GetFromNodeID: cannot retrieve adjacence info");
        return 0;
    }
    return Adj->GetFromNodeID();
}

// =================================================================================== 

inline TNode *
TEdge::GetOtherNode(TNode *aNode, TInfoID AdjacenceInfoID) const

{
    TAdjacenceInfo *Adj;

    if ((Adj = (TAdjacenceInfo *) GetInfo(DefAdj(AdjacenceInfoID), TRUE, TRUE)) == 0)
    {
        Error("GetFromNode: cannot retrieve adjacence info");
        return 0;
    }
    if (Adj->GetFromNode() == aNode)
        return Adj->GetToNode();
    if (Adj->GetToNode() == aNode)
        return Adj->GetFromNode();
    Warning("GetOtherNode: node not adjacent to edge");
    return 0;
}

// =================================================================================== 

inline TEdge *
TEdge::GetPtr(TParallelID ID, bool Warn)

{
    return (TEdge *) ClassInfo->GetObjectPtr(ID, Warn);
}

// =================================================================================== 

inline TNode *
TEdge::GetToNode(TInfoID AdjacenceInfoID) const

{
    TAdjacenceInfo *Adj;

    if ((Adj = (TAdjacenceInfo *) GetInfo(DefAdj(AdjacenceInfoID), TRUE, TRUE)) == 0)
    {
        Error("GetToNode: cannot retrieve adjacence info");
        return 0;
    }
    return Adj->GetToNode(FALSE);
}

// =================================================================================== 

inline TParallelID
TEdge::GetToNodeID(TInfoID AdjacenceInfoID) const

{
    TAdjacenceInfo *Adj;

    if ((Adj = (TAdjacenceInfo *) GetInfo(DefAdj(AdjacenceInfoID), TRUE, TRUE)) == 0)
    {
        Error("GetToNodeID: cannot retrieve adjacence info");
        return 0;
    }
    return Adj->GetToNodeID();
}

// =================================================================================== 

